--
-- Copyright (c) 2015 Aitia International Inc.
--
-- See the file "/doc/license.txt" for information on usage and redistribution
-- of this file, and for a DISCLAIMER OF ALL WARRANTIES.
-- 

LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_UNSIGNED.all;
USE work.tuw.all;

PACKAGE cg_framegen_pkg IS

COMPONENT CG_FrameGen IS
	PORT(
	clk 						: IN  STD_LOGIC;
	Rst 						: IN  STD_LOGIC;
	-- Config
	Cfg_Frlen					: IN  STD_LOGIC_VECTOR(16-1 downto 0); -- in bytes
	Cfg_Ifslen				: IN  STD_LOGIC_VECTOR(16-1 downto 0); -- IFS in 512bit vectors
--	Cfg_PckCnt				: IN  STD_LOGIC_VECTOR(32-1 downto 0); -- nr of packets
--	Cfg_Data					: IN  STD_LOGIC_VECTOR(64-1 downto 0); -- frame data
	Cfg_Wr					: IN  STD_LOGIC;
	--
	CG_TxAv					: IN  STD_LOGIC;
	CG_TXData 				: OUT STD_LOGIC_VECTOR(512-1 downto 0);
	CG_TXDv 					: OUT STD_LOGIC;
	CG_TXSof					: OUT STD_LOGIC;
	CG_TXEof					: OUT STD_LOGIC;
	CG_TXMod64				: OUT STD_LOGIC_VECTOR(6-1 downto 0)
	);
END COMPONENT;

COMPONENT CG_FrameGen_wBRAM IS
	PORT(
	clk_cfg					: IN  STD_LOGIC; -- PCIex clk
	clk_if 					: IN  STD_LOGIC; -- CG-Tx clk
	-- Config
	Cfg_Rst					: IN  STD_LOGIC;
	Cfg_Frlen					: IN  STD_LOGIC_VECTOR(16-1 downto 0); -- Framelength in bytes
	Cfg_Ifslen				: IN  STD_LOGIC_VECTOR(16-1 downto 0); -- IFS in 320bit vectors
	Cfg_PckCnt				: IN  STD_LOGIC_VECTOR(32-1 downto 0); -- nr of packets from type 1
	Cfg_BurstCnt				: IN  STD_LOGIC_VECTOR(32-1 downto 0); -- nr of packets+1 from type 2 (in a burst)
	Cfg_Data					: IN  STD_LOGIC_VECTOR(32-1 downto 0); -- frame data
	Cfg_Wr 					: IN  STD_LOGIC; -- write in data
	Cfg_WrSelect				: IN  STD_LOGIC; -- write frame selector
	Cfg_Ctrl					: IN  STD_LOGIC; -- arm the transmitter
	--
	CG_TxAv					: IN  STD_LOGIC;
	CG_TXData 				: OUT STD_LOGIC_VECTOR(512-1 downto 0);
	CG_TXDv 					: OUT STD_LOGIC;
	CG_TXSof					: OUT STD_LOGIC;
	CG_TXEof					: OUT STD_LOGIC;
	CG_TXMod64				: OUT STD_LOGIC_VECTOR(6-1 downto 0)
	);
END COMPONENT;

COMPONENT CG_FrameGen_wBRAMv2 IS
	PORT(
	clk_cfg					: IN  STD_LOGIC; -- PCIex clk
	clk_if 					: IN  STD_LOGIC; -- CG-Tx clk
	-- Config
	Cfg_Rst					: IN  STD_LOGIC;
	Cfg_Data					: IN  STD_LOGIC_VECTOR(32-1 downto 0); -- frame data
	Cfg_Wr 					: IN  STD_LOGIC; -- write in data
	-- Frame control
	FRCtrl_Frlen				: IN  STD_LOGIC_VECTOR(16-1 downto 0); -- Framelength in bytes
	FRCtrl_Ifslen				: IN  STD_LOGIC_VECTOR(16-1 downto 0); -- IFS in 320bit vectors
	FRCtrl_Select 				: IN  STD_LOGIC_VECTOR(8-1 downto 0); -- frame type selector
	FRCtrl_Arm				: IN  STD_LOGIC; -- (re)arm the transmitter
	FRCtrl_Sent				: OUT STD_LOGIC; -- frame sent (ready for next command)
	--
	CG_TxAv					: IN  STD_LOGIC;
	CG_TXData 				: OUT STD_LOGIC_VECTOR(512-1 downto 0);
	CG_TXDv 					: OUT STD_LOGIC;
	CG_TXSof					: OUT STD_LOGIC;
	CG_TXEof					: OUT STD_LOGIC;
	CG_TXMod64				: OUT STD_LOGIC_VECTOR(6-1 downto 0)
	);
END COMPONENT;

COMPONENT RAM_512entry_64b IS
	GENERIC ( DELAY	: INTEGER := 0;
			TABLE_VECTOR_1 : BIT_VECTOR;
			TABLE_VECTOR_2 : BIT_VECTOR);
	PORT (
	clk_Wr	: IN STD_LOGIC;
	clk_Rd	: IN STD_LOGIC;
	--
	WrEn 	: IN  STD_LOGIC;
	WrAddr 	: IN  STD_LOGIC_VECTOR(8+1-1 DOWNTO 0);
	RdAddr 	: IN  STD_LOGIC_VECTOR(8+1-1 DOWNTO 0);
	Din	 	: IN  STD_LOGIC_VECTOR(64-1 DOWNTO 0);
	Dout 	: OUT STD_LOGIC_VECTOR(64-1 DOWNTO 0)
	);
END COMPONENT; 

END PACKAGE;

library IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_UNSIGNED.ALL;
USE work.tuw.ALL;
USE work.arch.all;
USE work.cg_framegen_pkg.all;
USE work.cg_frdata_pkg.ALL;

library unisim;
use unisim.vcomponents.all;

ENTITY CG_FrameGen_wBRAM IS
	PORT(
	clk_cfg					: IN  STD_LOGIC; -- PCIex clk
	clk_if 					: IN  STD_LOGIC; -- CG-Tx clk
	-- Config
	Cfg_Rst					: IN  STD_LOGIC;
	Cfg_Frlen					: IN  STD_LOGIC_VECTOR(16-1 downto 0); -- Framelength in bytes
	Cfg_Ifslen				: IN  STD_LOGIC_VECTOR(16-1 downto 0); -- IFS in 320bit vectors
	Cfg_PckCnt				: IN  STD_LOGIC_VECTOR(32-1 downto 0); -- nr of packets from type 1
	Cfg_BurstCnt				: IN  STD_LOGIC_VECTOR(32-1 downto 0); -- nr of packets+1 from type 2 (in a burst)
	Cfg_Data					: IN  STD_LOGIC_VECTOR(32-1 downto 0); -- frame data
	Cfg_Wr 					: IN  STD_LOGIC; -- write in data
	Cfg_WrSelect				: IN  STD_LOGIC; -- write frame selector
	Cfg_Ctrl					: IN  STD_LOGIC; -- arm the transmitter
	--
	CG_TxAv					: IN  STD_LOGIC;
	CG_TXData 				: OUT STD_LOGIC_VECTOR(512-1 downto 0);
	CG_TXDv 					: OUT STD_LOGIC;
	CG_TXSof					: OUT STD_LOGIC;
	CG_TXEof					: OUT STD_LOGIC;
	CG_TXMod64				: OUT STD_LOGIC_VECTOR(6-1 downto 0)
	);
END CG_FrameGen_wBRAM;

ARCHITECTURE CG_FrameGen_wBRAM_arch OF CG_FrameGen_wBRAM IS

SIGNAL aCG_Ifslen 			: STD_LOGIC_VECTOR(16-1 downto 0);
SIGNAL CG_Ifslen_reg		: STD_LOGIC_VECTOR(16-1 downto 0);
SIGNAL aCG_Frlen 			: STD_LOGIC_VECTOR(16-6-1 downto 0);
SIGNAL CG_Frlen_reg			: STD_LOGIC_VECTOR(16-6-1 downto 0);
SIGNAL aCG_Mod40 			: STD_LOGIC_VECTOR(6-1 downto 0);
SIGNAL CG_Mod40_reg			: STD_LOGIC_VECTOR(6-1 downto 0);
SIGNAL aCG_Frcnt_select		: STD_LOGIC_VECTOR(32-1 downto 0);
SIGNAL CG_Frcnt_select_reg	: STD_LOGIC_VECTOR(32-1 downto 0);
SIGNAL aCG_Frcnt_burst		: STD_LOGIC_VECTOR(32-1 downto 0);
SIGNAL CG_Frcnt_burst_reg	: STD_LOGIC_VECTOR(32-1 downto 0);


SIGNAL xCfg_Ctrl			: STD_LOGIC;
SIGNAL dxCfg_Ctrl			: STD_LOGIC;
SIGNAL dCfg_Rst			: STD_LOGIC;
SIGNAL dCG_TxAv			: STD_LOGIC;

SIGNAL Wr_Even				: STD_LOGIC;
SIGNAL Wr_Select			: STD_LOGIC_VECTOR(5-1 downto 0);
SIGNAL FR_WrEn				: STD_LOGIC_VECTOR(5-1 downto 0);
SIGNAL wrdta_cntr			: STD_LOGIC_VECTOR(16-6-1 downto 0) := (others => '0');

SIGNAL aCG_TXSof 			: STD_LOGIC;
SIGNAL aCG_TXEof 			: STD_LOGIC;
SIGNAL aCG_TXDv 			: STD_LOGIC;
SIGNAL aCG_TXMod64 			: STD_LOGIC_VECTOR(6-1 downto 0);
SIGNAL daCG_TXSof 			: STD_LOGIC;
SIGNAL daCG_TXEof 			: STD_LOGIC;
SIGNAL daCG_TXDv 			: STD_LOGIC;
SIGNAL daCG_TXMod64 		: STD_LOGIC_VECTOR(6-1 downto 0);

SIGNAL aCGTH_txdata	 		: STD_LOGIC_VECTOR(64-1 downto 0);

TYPE tyTX_State			IS (TX_RESET, TX_IFS, TX_SOF, TX_DATA, TX_EOF);
SIGNAL TX_State			: tyTX_State := TX_RESET;

SIGNAL ifs_cntr			: STD_LOGIC_VECTOR(16-1 downto 0) := (others => '0');
SIGNAL rddta_cntr			: STD_LOGIC_VECTOR(16-6-1 downto 0) := (others => '0');
SIGNAL fr_cntr				: STD_LOGIC_VECTOR(32-1 downto 0) := (others => '0');
SIGNAL fr_2ndcnt			: STD_LOGIC_VECTOR(32-1 downto 0) := (others => '0');

SIGNAL FR_IFSDone			: STD_LOGIC;
SIGNAL FR_DTADone			: STD_LOGIC;

SIGNAL CG_Idle_was			: STD_LOGIC;
SIGNAL TxCfg_accepted		: STD_LOGIC;
SIGNAL dTxCfg_accepted		: STD_LOGIC;
SIGNAL TxCfg_wrwas			: STD_LOGIC := '0';

SIGNAL FR_RdAddr 			: STD_LOGIC_VECTOR(8+1-1 downto 0);
SIGNAL FR_RdData 			: STD_LOGIC_VECTOR(5*64-1 downto 0);
SIGNAL FR_RdSelect 			: STD_LOGIC := '0';
SIGNAL FR_2ndEnd 			: STD_LOGIC := '0';

SIGNAL FR_WrAddr 			: STD_LOGIC_VECTOR(8+1-1 downto 0);
SIGNAL FR_WrData 			: STD_LOGIC_VECTOR(1*64-1 downto 0);
SIGNAL FR_WrSelect 			: STD_LOGIC := '0';

BEGIN

--
---- 1. Transmit Config and control
--

Ucfgx: aRippleRise PORT MAP( I => Cfg_Ctrl, C => clk_if, Q => xCfg_Ctrl);

UTXCtrl: PROCESS(clk_if, dxCfg_Ctrl, TxCfg_accepted)
BEGIN
IF (clk_if'event AND clk_if = '1') THEN
--
dCfg_Rst			<= Cfg_Rst;
dCG_TxAv			<= CG_TxAv;
dxCfg_Ctrl		<= xCfg_Ctrl;
dTxCfg_accepted	<= TxCfg_accepted;

IF (dxCfg_Ctrl = '1') THEN
	aCG_Mod40 			<= Cfg_Frlen(6-1 downto 0);
	aCG_Frlen 			<= Cfg_Frlen(16-1 downto 6);
	aCG_Ifslen			<= Cfg_Ifslen;
	aCG_Frcnt_select		<= Cfg_Pckcnt;
	aCG_Frcnt_burst		<= Cfg_Burstcnt;
	TxCfg_wrwas			<= '1';
ELSIF (TxCfg_accepted = '1') THEN
	TxCfg_wrwas 			<= '0';
END IF;
--
IF (TxCfg_accepted = '1') THEN
	CG_Mod40_reg			<= aCG_Mod40;
	CG_Frlen_reg			<= aCG_Frlen - 1;
	CG_IFslen_reg 			<= aCG_IFslen;
	CG_Frcnt_select_reg		<= aCG_Frcnt_select;
	CG_Frcnt_burst_reg		<= aCG_Frcnt_burst;
END IF;
--
END IF;
END PROCESS;


--
---- 2. Write in frame data
--

UTXDta: PROCESS(clk_cfg, Cfg_Rst, Cfg_Wr, Wr_Even)
BEGIN
IF (clk_cfg'event AND clk_cfg = '1') THEN
--
Fr_WrSelect 	<= Cfg_WrSelect;
--
IF (Cfg_Rst = '1') THEN
	wrdta_cntr	<= (others => '1'); -- first increment sets 0 wr_address
	Wr_Select		<= "00001";
	Wr_Even		<= '0';
	FR_WrEn		<= "00000";	
ELSIF (Cfg_Wr = '1') THEN
	Wr_Even		<= NOT Wr_Even;
	IF (Wr_Select(0) = '1' AND Wr_Even = '0') THEN -- incr address upon first odd nibble
		INCR(wrdta_cntr);
	END IF;
	IF (Wr_Even = '1') THEN
		FR_WrData(2*32-1 downto 1*32)		<= Cfg_Data;
		FR_WrEn 						<= Wr_Select;
		Wr_Select						<= Wr_Select(5-2 downto 0) & Wr_Select(5-1);
	ELSE
		FR_WrData(1*32-1 downto 0*32)		<= Cfg_Data;	
		FR_WrEn 						<= "00000";
	END IF;
ELSE
	FR_WrEn			<= "00000";
END IF;
--
END IF;
END PROCESS;

FR_WrAddr		<= "000" & FR_WrSelect & wrdta_cntr(5-1 downto 0);
FR_RdAddr		<= "000" & FR_RdSelect & rddta_cntr(5-1 downto 0);

-- RAMs containing the frames to send
UFrg: FOR I IN 0 TO (5-1) GENERATE
BEGIN

Ug: RAM_512entry_64b 
	GENERIC MAP( DELAY	=> 1,
			TABLE_VECTOR_1 => frdata_tables(I),
			TABLE_VECTOR_2 => DATA_NULL_VECTOR 
			)
	PORT MAP(
	clk_Rd	=> clk_if,
	clk_Wr	=> clk_cfg,
	--
	WrEn		=> FR_WrEn(I),
	WrAddr	=> FR_WrAddr,
	RdAddr 	=> FR_RdAddr,
	Din		=> FR_WrData,
	Dout 	=> FR_RdData(64*(I+1)-1 downto 64*I)
	);

END GENERATE;

--
---- 3. CG Trafficgen :)
--

UTXgen: PROCESS(clk_if, dCfg_Rst, TX_State, fr_cntr, fr_2ndcnt, ifs_cntr, rddta_cntr, CG_Frcnt_select_reg, dTxCfg_accepted, dCG_TxAv, CG_Ifslen_reg, CG_Frlen_reg, Fr_RdSelect, FR_2ndEnd, TxCfg_wrwas)
BEGIN
IF (clk_if'event AND clk_if = '1') THEN
--
daCG_TXSof 			<= aCG_TXSof;
daCG_TXEof 			<= aCG_TXEof;
daCG_TXDv 			<= aCG_TXDv;
daCG_TXMod64 			<= aCG_TXMod64;

CG_TXData(5*64-1 downto 0) 	<= FR_RdData;
CG_TXSof 					<= daCG_TXSof;
CG_TXEof 					<= daCG_TXEof;
CG_TXDv 					<= daCG_TXDv;
CG_TXMod64 				<= daCG_TXMod64;

--
IF (fr_cntr = CG_Frcnt_select_reg) THEN
	FR_RdSelect		<= '1';
ELSE
	FR_RdSelect		<= '0';
END IF;
--
IF (fr_2ndcnt = CG_Frcnt_burst_reg) THEN -- Burst length of type 2 frames!
	FR_2ndEnd		<= '1';
ELSE
	FR_2ndEnd		<= '0';
END IF;

IF (ifs_cntr = CG_Ifslen_reg) THEN
	FR_IFSDone	<= '1';
ELSE
	FR_IFSDone	<= '0';
END IF;
IF (rddta_cntr = CG_Frlen_reg) THEN
	FR_DTADone	<= '1';
ELSE
	FR_DTADone	<= '0';
END IF;
--
	CASE TX_State IS
		WHEN TX_RESET =>
			aCG_TXSof			<= '0';
			aCG_TXEof			<= '0';
			aCG_TXDv			<= '0';
			aCG_TXMod64		<= "000000";
			RESET(fr_cntr);
			RESET(fr_2ndcnt);			
			TxCfg_accepted		<= TxCfg_wrwas;
			IF (dTxCfg_accepted = '1') THEN
				TX_State			<= TX_IFS;
			END IF;
		WHEN TX_IFS => -- IFS: one minimum for now!
			aCG_TXSof			<= '0';
			aCG_TXEof			<= '0';
			aCG_TXDv			<= '0';
			aCG_TXMod64		<= "000000";		
			RESET(rddta_cntr);
			TxCfg_accepted		<= '0';			
			CG_Idle_was		<= '1';
			IF (dCG_TxAv = '1') THEN
				INCR(ifs_cntr);
				IF (FR_IFSDone = '1') THEN
					TX_State		<= TX_DATA; --TX_SOF;
				END IF;
			END IF;
		-- WHEN TX_SOF => -- SOF
		-- 	CG_TXSof			<= '1';
		-- 	CG_TXEof			<= '0';
		-- 	CG_TXDv			<= '1';
		-- 	TX_State			<= TX_DATA;
		-- 	INCR(rddta_cntr);
		-- 	RESET(ifs_cntr);
		WHEN TX_DATA => -- DATA
			CG_Idle_was		<= '0';
			aCG_TXSof			<= CG_Idle_was;
			aCG_TXEof			<= '0';
			aCG_TXDv			<= '1';
			INCR(rddta_cntr);
			IF (FR_DTADone = '1') THEN 
				TX_State		<= TX_EOF;
			END IF;
		WHEN TX_EOF => -- EOF
			aCG_TXSof			<= '0';
			aCG_TXEof			<= '1';
			aCG_TXDv			<= '1';
			aCG_TXMod64		<= CG_Mod40_reg;
			RESET(ifs_cntr);
			--
			IF (FR_RdSelect = '1') THEN
				IF (FR_2ndEnd = '1') THEN
					RESET(fr_2ndcnt);
					RESET(fr_cntr);
				ELSE
					INCR(fr_2ndcnt);				
				END IF;
			ELSE
				INCR(fr_cntr);
			END IF;
			--
			IF (dCfg_Rst = '1' OR TxCfg_wrwas = '1') THEN
				TX_State			<= TX_RESET;
			ELSE
				TX_State			<= TX_IFS;
			END IF;
		WHEN OTHERS =>
			NULL;
	END CASE;
END IF;
END PROCESS; 

END CG_FrameGen_wBRAM_arch;


library IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_UNSIGNED.ALL;
USE work.tuw.ALL;
USE work.arch.all;
USE work.cg_framegen_pkg.all;
USE work.cg_frdata_pkg.ALL;

library unisim;
use unisim.vcomponents.all;

ENTITY CG_FrameGen_wBRAMv2 IS
	PORT(
	clk_cfg					: IN  STD_LOGIC; -- PCIex clk
	clk_if 					: IN  STD_LOGIC; -- CG-Tx clk
	-- Config
	Cfg_Rst					: IN  STD_LOGIC;
	Cfg_Data					: IN  STD_LOGIC_VECTOR(32-1 downto 0); -- frame data
	Cfg_Wr 					: IN  STD_LOGIC; -- write in data
	-- Frame control
	FRCtrl_Frlen				: IN  STD_LOGIC_VECTOR(16-1 downto 0); -- Framelength in bytes
	FRCtrl_Ifslen				: IN  STD_LOGIC_VECTOR(16-1 downto 0); -- IFS in 320bit vectors
	FRCtrl_Select 				: IN  STD_LOGIC_VECTOR(8-1 downto 0); -- frame type selector
	FRCtrl_Arm				: IN  STD_LOGIC; -- (re)arm the transmitter
	FRCtrl_Sent				: OUT STD_LOGIC; -- frame sent (ready for next command)
	--
	CG_TxAv					: IN  STD_LOGIC;
	CG_TXData 				: OUT STD_LOGIC_VECTOR(512-1 downto 0);
	CG_TXDv 					: OUT STD_LOGIC;
	CG_TXSof					: OUT STD_LOGIC;
	CG_TXEof					: OUT STD_LOGIC;
	CG_TXMod64				: OUT STD_LOGIC_VECTOR(6-1 downto 0)
	);
END CG_FrameGen_wBRAMv2;

ARCHITECTURE CG_FrameGen_wBRAMv2_arch OF CG_FrameGen_wBRAMv2 IS

SIGNAL CG_Select 			: STD_LOGIC_VECTOR(8-1 downto 0);
SIGNAL CG_Ifslen			: STD_LOGIC_VECTOR(16-1 downto 0);
SIGNAL CG_Frlen			: STD_LOGIC_VECTOR(6-1 downto 0);
SIGNAL CG_Mod40			: STD_LOGIC_VECTOR(6-1 downto 0);
SIGNAL dCG_Mod40			: STD_LOGIC_VECTOR(6-1 downto 0);
SIGNAL dFRCtrl_Select		: STD_LOGIC_VECTOR(8-1 downto 0);

SIGNAL dCfg_Rst			: STD_LOGIC;
SIGNAL d1CG_TxAv			: STD_LOGIC;
SIGNAL d2CG_TxAv			: STD_LOGIC;

SIGNAL Wr_Even				: STD_LOGIC;
SIGNAL Wr_Select			: STD_LOGIC_VECTOR(5-1 downto 0);
SIGNAL FR_WrEn				: STD_LOGIC_VECTOR(5-1 downto 0);
SIGNAL wrdta_cntr			: STD_LOGIC_VECTOR(16-6-1 downto 0) := (others => '0');

SIGNAL aCG_TXSof 			: STD_LOGIC;
SIGNAL aCG_TXEof 			: STD_LOGIC;
SIGNAL aCG_TXDv 			: STD_LOGIC;
SIGNAL aCG_TXMod64 			: STD_LOGIC_VECTOR(6-1 downto 0);
SIGNAL daCG_TXSof 			: STD_LOGIC;
SIGNAL daCG_TXEof 			: STD_LOGIC;
SIGNAL daCG_TXDv 			: STD_LOGIC;
SIGNAL daCG_TXMod64 		: STD_LOGIC_VECTOR(6-1 downto 0);

SIGNAL aCGTH_txdata	 		: STD_LOGIC_VECTOR(64-1 downto 0);

TYPE tyTX_State			IS (TX_RESET, TX_IFS, TX_SOF, TX_DATA, TX_EOF);
SIGNAL TX_State			: tyTX_State := TX_RESET;

SIGNAL ifs_cntr			: STD_LOGIC_VECTOR(16-1 downto 0) := (others => '0');
SIGNAL rddta_cntr			: STD_LOGIC_VECTOR(6-1 downto 0) := (others => '0');

SIGNAL FR_Sending			: STD_LOGIC;
SIGNAL FR_IFSDone			: STD_LOGIC;
SIGNAL FR_DTADone			: STD_LOGIC;

SIGNAL CG_Idle_was			: STD_LOGIC;
SIGNAL TxCfg_accepted		: STD_LOGIC;
SIGNAL dTxCfg_accepted		: STD_LOGIC;
SIGNAL TxCfg_wrwas			: STD_LOGIC := '0';

SIGNAL FR_RdAddr 			: STD_LOGIC_VECTOR(8+1-1 downto 0);
SIGNAL FR_RdData 			: STD_LOGIC_VECTOR(5*64-1 downto 0);
SIGNAL FR_RdSelect 			: STD_LOGIC_VECTOR(9-2-1 downto 0);

SIGNAL FR_WrAddr 			: STD_LOGIC_VECTOR(8+1-1 downto 0);
SIGNAL FR_WrData 			: STD_LOGIC_VECTOR(1*64-1 downto 0);
SIGNAL FR_WrSelect 			: STD_LOGIC := '0';

BEGIN

--
---- 1. Transmit Config and control
--

UTXCtrl: PROCESS(clk_if, FRCtrl_Arm, FR_DTADone )
BEGIN
IF (clk_if'event AND clk_if = '1') THEN
--
dCfg_Rst			<= Cfg_Rst;
d1CG_TxAv			<= CG_TxAv;
d2CG_TxAv			<= d1CG_TxAv;
TxCfg_accepted		<= FRCtrl_Arm;

dCG_Mod40 		<= CG_Mod40;
dFRCtrl_Select		<= FRCtrl_Select;
--
END IF;
END PROCESS;

	CG_Ifslen			<= FRCtrl_Ifslen; -- IFS is delayed for the next packet
	CG_Mod40 			<= FRCtrl_Frlen(6-1 downto 0);
	CG_Frlen 			<= FRCtrl_Frlen(12-1 downto 6);
	CG_Select			<= dFRCtrl_Select;

--
---- 2. Write in frame data
--

UTXDta: PROCESS(clk_cfg, Cfg_Rst, Cfg_Wr, Wr_Even)
BEGIN
IF (clk_cfg'event AND clk_cfg = '1') THEN
--
IF (Cfg_Rst = '1') THEN
	wrdta_cntr	<= (others => '1'); -- first increment sets 0 wr_address
	Wr_Select		<= "00001";
	Wr_Even		<= '0';
	FR_WrEn		<= "00000";	
ELSIF (Cfg_Wr = '1') THEN
	Wr_Even		<= NOT Wr_Even;
	IF (Wr_Select(0) = '1' AND Wr_Even = '0') THEN -- incr address upon first odd nibble
		INCR(wrdta_cntr);
	END IF;
	IF (Wr_Even = '1') THEN
		FR_WrData(2*32-1 downto 1*32)		<= Cfg_Data;
		FR_WrEn 						<= Wr_Select;
		Wr_Select						<= Wr_Select(5-2 downto 0) & Wr_Select(5-1);
	ELSE
		FR_WrData(1*32-1 downto 0*32)		<= Cfg_Data;	
		FR_WrEn 						<= "00000";
	END IF;
ELSE
	FR_WrEn			<= "00000";
END IF;
--
END IF;
END PROCESS;

FR_WrAddr		<= wrdta_cntr(9-1 downto 0);
FR_RdAddr		<= FR_RdSelect & rddta_cntr(2-1 downto 0);

FR_RdSelect	<= CG_Select(9-2-1 downto 0);

-- RAMs containing the frames to send
UFrg: FOR I IN 0 TO (5-1) GENERATE
BEGIN

Ug: RAM_512entry_64b 
	GENERIC MAP( DELAY	=> 1,
			TABLE_VECTOR_1 => frdata_tables(I),
			TABLE_VECTOR_2 => DATA_NULL_VECTOR 
			)
	PORT MAP(
	clk_Rd	=> clk_if,
	clk_Wr	=> clk_cfg,
	--
	WrEn		=> FR_WrEn(I),
	WrAddr	=> FR_WrAddr,
	RdAddr 	=> FR_RdAddr,
	Din		=> FR_WrData,
	Dout 	=> FR_RdData(64*(I+1)-1 downto 64*I)
	);

END GENERATE;

--
---- 3. CG Trafficgen :)
--

FRCtrl_Sent	<= FR_DTADone AND FR_Sending;

UTXgen: PROCESS(clk_if, dCfg_Rst, TX_State, ifs_cntr, rddta_cntr, d1CG_TxAv, CG_Ifslen, CG_Frlen )
BEGIN
IF (clk_if'event AND clk_if = '1') THEN
--
CG_TXData(5*64-1 downto 0) 	<= FR_RdData;
CG_TXSof 					<= aCG_TXSof;
CG_TXEof 					<= aCG_TXEof;
CG_TXDv 					<= aCG_TXDv;
CG_TXMod64 				<= aCG_TXMod64;
--
IF (ifs_cntr = CG_Ifslen) THEN
	FR_IFSDone	<= '1';
ELSE
	FR_IFSDone	<= '0';
END IF;
IF (rddta_cntr = CG_Frlen) THEN
	FR_DTADone	<= '1';
ELSE
	FR_DTADone	<= '0';
END IF;
--
	CASE TX_State IS
		WHEN TX_RESET =>
			aCG_TXSof			<= '0';
			aCG_TXEof			<= '0';
			aCG_TXDv			<= '0';
			aCG_TXMod64		<= "000000";
			RESET(rddta_cntr);
			RESET(ifs_cntr);
			FR_Sending		<= '0';
			IF (TxCfg_accepted = '1') THEN
				TX_State			<= TX_IFS;
			END IF;
		WHEN TX_IFS => -- IFS: one minimum for now!
			aCG_TXSof			<= '0';
			aCG_TXEof			<= '0';
			aCG_TXDv			<= '0';
			aCG_TXMod64		<= "000000";		
			CG_Idle_was		<= '1';
			IF (d2CG_TxAv = '1') THEN
				INCR(ifs_cntr);
				IF (FR_IFSDone = '1') THEN
					TX_State		<= TX_DATA; --TX_SOF;
					rddta_cntr(0)	<= '1'; -- pre-start counter
					FR_Sending	<= '1';
				END IF;
			END IF;
		WHEN TX_DATA => -- DATA
			CG_Idle_was		<= '0';
			aCG_TXSof			<= CG_Idle_was;
			aCG_TXEof			<= '0';
			aCG_TXDv			<= '1';
			INCR(rddta_cntr);
			RESET(ifs_cntr);
			IF (FR_DTADone = '1') THEN 
				TX_State		<= TX_EOF;
				FR_Sending	<= '0';
--			ELSIF FR_HDRDone = '1') THEN
--				TX_State		<= TX_FILLER;
			END IF;
--		WHEN TX_FILLER => -- Rest of frame (PRBS)
--			aCG_TXSof			<= '0';
--			aCG_TXEof			<= '0';
--			aCG_TXDv			<= '1';
--			IF (FR_DTADone = '1') THEN 
--				TX_State		<= TX_EOF;
--			END IF;
		WHEN TX_EOF => -- EOF
			aCG_TXSof			<= '0';
			aCG_TXEof			<= '1';
			aCG_TXDv			<= '1';
			aCG_TXMod64		<= dCG_Mod40;
			RESET(rddta_cntr);
			FR_Sending		<= '0';
			ifs_cntr(0)		<= d1CG_TxAv; -- pre-start only if FIFO is available
			--
			IF (dCfg_Rst = '1') THEN
				TX_State			<= TX_RESET;
			ELSE
				TX_State			<= TX_IFS;
			END IF;
		WHEN OTHERS =>
			NULL;
	END CASE;
END IF;
END PROCESS; 

END CG_FrameGen_wBRAMv2_arch;



LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;


library UNISIM;
use UNISIM.VCOMPONENTS.ALL;

-- synthesis translate_off
-- synthesis translate_on
ENTITY RAM_512entry_64b IS
	GENERIC ( DELAY	: INTEGER;	
			TABLE_VECTOR_1 : BIT_VECTOR;
			TABLE_VECTOR_2 : BIT_VECTOR);
	PORT (
	clk_Wr	: IN STD_LOGIC;
	clk_Rd	: IN STD_LOGIC;
	--
	WrEn 	: IN  STD_LOGIC;
	WrAddr 	: IN  STD_LOGIC_VECTOR(8+1-1 DOWNTO 0);
	RdAddr 	: IN  STD_LOGIC_VECTOR(8+1-1 DOWNTO 0);
	Din	 	: IN  STD_LOGIC_VECTOR(64-1 DOWNTO 0);
	Dout 	: OUT STD_LOGIC_VECTOR(64-1 DOWNTO 0)
	);
END RAM_512entry_64b;

ARCHITECTURE arch_RAM_512entry_64b OF RAM_512entry_64b IS

SIGNAL DOADO 			: STD_LOGIC_VECTOR(4*8-1 downto 0);
SIGNAL DOBDO 			: STD_LOGIC_VECTOR(4*8-1 downto 0);
SIGNAL ADDRARDADDR		: STD_LOGIC_VECTOR(16-1 downto 0) := (others => '0');
SIGNAL ADDRBWRADDR		: STD_LOGIC_VECTOR(16-1 downto 0) := (others => '0');
SIGNAL WEBWE			: STD_LOGIC_VECTOR(8-1 downto 0);
SIGNAL nWREN			: STD_LOGIC;

BEGIN

Dout(4*8-1 downto 0*8)	<= DOADO;
Dout(8*8-1 downto 4*8)	<= DOBDO;

ADDRARDADDR(15-1 downto 15-9)	<= RdAddr;
ADDRBWRADDR(15-1 downto 15-9)	<= WrAddr;

WEBWE		<= WrEn & WrEn & WrEn & WrEn & WrEn & WrEn & WrEn & WrEn;
nWREN		<= NOT WrEn;

-- Configuration specification
  RAMB36E1_inst : RAMB36E1
	GENERIC MAP (
	DOA_REG 				=> DELAY,
	DOB_REG 				=> DELAY,
	INIT_FILE 			=> "NONE", --INIT_FILE,
	-- Table 1
	INIT_00				=> TABLE_VECTOR_1(1*256-1 downto 0*256),
	INIT_01				=> TABLE_VECTOR_1(2*256-1 downto 1*256),
	INIT_02				=> TABLE_VECTOR_1(3*256-1 downto 2*256),
	INIT_03				=> TABLE_VECTOR_1(4*256-1 downto 3*256),
	INIT_04				=> TABLE_VECTOR_1(5*256-1 downto 4*256),
	INIT_05				=> TABLE_VECTOR_1(6*256-1 downto 5*256),
	INIT_06				=> TABLE_VECTOR_1(7*256-1 downto 6*256),
	INIT_07				=> TABLE_VECTOR_1(8*256-1 downto 7*256),
	INIT_08				=> TABLE_VECTOR_1(9*256-1 downto 8*256),
	INIT_09				=> TABLE_VECTOR_1(10*256-1 downto 9*256),
	INIT_0A				=> TABLE_VECTOR_1(11*256-1 downto 10*256),
	INIT_0B				=> TABLE_VECTOR_1(12*256-1 downto 11*256),
	INIT_0C				=> TABLE_VECTOR_1(13*256-1 downto 12*256),
	INIT_0D				=> TABLE_VECTOR_1(14*256-1 downto 13*256),
	INIT_0E				=> TABLE_VECTOR_1(15*256-1 downto 14*256),
	INIT_0F				=> TABLE_VECTOR_1(16*256-1 downto 15*256),
	INIT_10				=> TABLE_VECTOR_1(17*256-1 downto 16*256),
	INIT_11				=> TABLE_VECTOR_1(18*256-1 downto 17*256),
	INIT_12				=> TABLE_VECTOR_1(19*256-1 downto 18*256),
	INIT_13				=> TABLE_VECTOR_1(20*256-1 downto 19*256),
	INIT_14				=> TABLE_VECTOR_1(21*256-1 downto 20*256),
	INIT_15				=> TABLE_VECTOR_1(22*256-1 downto 21*256),
	INIT_16				=> TABLE_VECTOR_1(23*256-1 downto 22*256),
	INIT_17				=> TABLE_VECTOR_1(24*256-1 downto 23*256),
	INIT_18				=> TABLE_VECTOR_1(25*256-1 downto 24*256),
	INIT_19				=> TABLE_VECTOR_1(26*256-1 downto 25*256),
	INIT_1A				=> TABLE_VECTOR_1(27*256-1 downto 26*256),
	INIT_1B				=> TABLE_VECTOR_1(28*256-1 downto 27*256),
	INIT_1C				=> TABLE_VECTOR_1(29*256-1 downto 28*256),
	INIT_1D				=> TABLE_VECTOR_1(30*256-1 downto 29*256),
	INIT_1E				=> TABLE_VECTOR_1(31*256-1 downto 30*256),
	INIT_1F				=> TABLE_VECTOR_1(32*256-1 downto 31*256),
	-- Table 2
	INIT_20				=> TABLE_VECTOR_2(1*256-1 downto 0*256),
	INIT_21				=> TABLE_VECTOR_2(2*256-1 downto 1*256),
	INIT_22				=> TABLE_VECTOR_2(3*256-1 downto 2*256),
	INIT_23				=> TABLE_VECTOR_2(4*256-1 downto 3*256),
	INIT_24				=> TABLE_VECTOR_2(5*256-1 downto 4*256),
	INIT_25				=> TABLE_VECTOR_2(6*256-1 downto 5*256),
	INIT_26				=> TABLE_VECTOR_2(7*256-1 downto 6*256),
	INIT_27				=> TABLE_VECTOR_2(8*256-1 downto 7*256),
	INIT_28				=> TABLE_VECTOR_2(9*256-1 downto 8*256),
	INIT_29				=> TABLE_VECTOR_2(10*256-1 downto 9*256),
	INIT_2A				=> TABLE_VECTOR_2(11*256-1 downto 10*256),
	INIT_2B				=> TABLE_VECTOR_2(12*256-1 downto 11*256),
	INIT_2C				=> TABLE_VECTOR_2(13*256-1 downto 12*256),
	INIT_2D				=> TABLE_VECTOR_2(14*256-1 downto 13*256),
	INIT_2E				=> TABLE_VECTOR_2(15*256-1 downto 14*256),
	INIT_2F				=> TABLE_VECTOR_2(16*256-1 downto 15*256),
	INIT_30				=> TABLE_VECTOR_2(17*256-1 downto 16*256),
	INIT_31				=> TABLE_VECTOR_2(18*256-1 downto 17*256),
	INIT_32				=> TABLE_VECTOR_2(19*256-1 downto 18*256),
	INIT_33				=> TABLE_VECTOR_2(20*256-1 downto 19*256),
	INIT_34				=> TABLE_VECTOR_2(21*256-1 downto 20*256),
	INIT_35				=> TABLE_VECTOR_2(22*256-1 downto 21*256),
	INIT_36				=> TABLE_VECTOR_2(23*256-1 downto 22*256),
	INIT_37				=> TABLE_VECTOR_2(24*256-1 downto 23*256),
	INIT_38				=> TABLE_VECTOR_2(25*256-1 downto 24*256),
	INIT_39				=> TABLE_VECTOR_2(26*256-1 downto 25*256),
	INIT_3A				=> TABLE_VECTOR_2(27*256-1 downto 26*256),
	INIT_3B				=> TABLE_VECTOR_2(28*256-1 downto 27*256),
	INIT_3C				=> TABLE_VECTOR_2(29*256-1 downto 28*256),
	INIT_3D				=> TABLE_VECTOR_2(30*256-1 downto 29*256),
	INIT_3E				=> TABLE_VECTOR_2(31*256-1 downto 30*256),
	INIT_3F				=> TABLE_VECTOR_2(32*256-1 downto 31*256),
	--
	RAM_MODE 				=> "SDP",
	READ_WIDTH_A 			=> 72,
	READ_WIDTH_B 			=> 0,
	WRITE_WIDTH_A 			=> 0,
	WRITE_WIDTH_B 			=> 72,
	WRITE_MODE_A 			=> "WRITE_FIRST",
	WRITE_MODE_B 			=> "WRITE_FIRST",
	RSTREG_PRIORITY_A 		=> "RSTREG",
	RSTREG_PRIORITY_B 		=> "RSTREG",
	SIM_COLLISION_CHECK 	=> "ALL"
	)
	PORT MAP (
	CLKARDCLK 			=> clk_Rd,
	CLKBWRCLK 			=> clk_Wr,
	--
	WEA 					=> (others => '0'),
	WEBWE 				=> WEBWE,
	ENBWREN 				=> '1', --ENBWREN,
	DIADI 				=> Din(4*8-1 downto 0*8), --DIADI,
	DIBDI 				=> Din(8*8-1 downto 4*8), --DIBDI,
	DIPADIP 				=> (others => '0'), --DIPADIP,
	DIPBDIP 				=> (others => '0'), --DIPBDIP,
	ADDRBWRADDR 			=> ADDRBWRADDR,
	CASCADEINA			=> '0',
	CASCADEINB			=> '0',
	injectdbiterr			=> '0',
	INJECTSBITERR			=> '0',
	--
	ENARDEN 				=> nWREN, --'1', --just to avoid simulation warnings, can be tied to '1' in practice
	DOADO 				=> DOADO,
	DOBDO 				=> DOBDO,
	DOPADOP 				=> open, --DOPADOP,
	DOPBDOP 				=> open, --DOPBDOP,
	ADDRARDADDR 			=> ADDRARDADDR,
	--
	REGCEAREGCE 			=> '1', --REGCEAREGCE,
	REGCEB 				=> '1', --REGCEB,
	RSTRAMARSTRAM			=> '0', 
	RSTRAMB				=> '0',
	RSTREGARSTREG			=> '0', 
	RSTREGB				=> '0'
);

END arch_RAM_512entry_64b;

library IEEE;
use IEEE.std_logic_1164.all;
USE IEEE.std_logic_UNSIGNED.ALL;
use work.tuw.all;
use work.arch.all;

library unisim;
use unisim.vcomponents.all;

ENTITY CG_FrameGen IS
	PORT(
	clk 						: IN  STD_LOGIC;
	Rst 						: IN  STD_LOGIC;
	-- Config
	Cfg_Frlen					: IN  STD_LOGIC_VECTOR(16-1 downto 0); -- in bytes
	Cfg_Ifslen				: IN  STD_LOGIC_VECTOR(16-1 downto 0); -- IFS in 512bit vectors
--	Cfg_PckCnt				: IN  STD_LOGIC_VECTOR(32-1 downto 0); -- nr of packets
--	Cfg_Data					: IN  STD_LOGIC_VECTOR(64-1 downto 0); -- frame data
	Cfg_Wr					: IN  STD_LOGIC;
	--
	CG_TxAv					: IN  STD_LOGIC;
	CG_TXData 				: OUT STD_LOGIC_VECTOR(512-1 downto 0);
	CG_TXDv 					: OUT STD_LOGIC;
	CG_TXSof					: OUT STD_LOGIC;
	CG_TXEof					: OUT STD_LOGIC;
	CG_TXMod64				: OUT STD_LOGIC_VECTOR(6-1 downto 0)
	);
END CG_FrameGen;

ARCHITECTURE CG_FrameGen_arch OF CG_FrameGen IS

SIGNAL aCG_Ifslen 			: STD_LOGIC_VECTOR(16-1 downto 0);
SIGNAL CG_Ifslen_reg		: STD_LOGIC_VECTOR(16-1 downto 0);
SIGNAL aCG_Frlen 			: STD_LOGIC_VECTOR(16-6-1 downto 0);
SIGNAL CG_Frlen_reg			: STD_LOGIC_VECTOR(16-6-1 downto 0);
SIGNAL aCG_Mod40 			: STD_LOGIC_VECTOR(6-1 downto 0);
SIGNAL CG_Mod40_reg			: STD_LOGIC_VECTOR(6-1 downto 0);

SIGNAL TxCfg_accepted		: STD_LOGIC;
SIGNAL dTxCfg_accepted		: STD_LOGIC;
SIGNAL TxCfg_wrwas			: STD_LOGIC := '0';
SIGNAL IFS_Extend			: STD_LOGIC;
SIGNAL IFS_Zero			: STD_LOGIC;
SIGNAL IFS_Zero_4real		: STD_LOGIC := '0';
SIGNAL PCK_Done			: STD_LOGIC;


SIGNAL aCGTH_txdata	 		: STD_LOGIC_VECTOR(64-1 downto 0) := (others => '0');
SIGNAL aCG_TXSof			: STD_LOGIC;
SIGNAL aCG_TXEof			: STD_LOGIC;
SIGNAL aCG_TXDv			: STD_LOGIC;
SIGNAL aCG_TXMod64			: STD_LOGIC_VECTOR(6-1 downto 0);

TYPE tyTX_State			IS (TX_CONFIG, TX_IFS, TX_SOF, TX_DATA);
SIGNAL TX_State			: tyTX_State := TX_CONFIG;

SIGNAL xCfg_Wr				: STD_LOGIC;
SIGNAL dxCfg_Wr			: STD_LOGIC;
SIGNAL dRst				: STD_LOGIC;
SIGNAL dCG_TxAv			: STD_LOGIC;
SIGNAL ifs_cntr			: STD_LOGIC_VECTOR(16-1 downto 0) := (others => '0');
SIGNAL dta_cntr			: STD_LOGIC_VECTOR(16-6-1 downto 0) := (others => '0');
SIGNAL pck_cntr			: STD_LOGIC_VECTOR(32-1 downto 0) := (others => '0');

BEGIN

aCGTH_txdata(2*32-1 downto 1*32)		<= aCGTH_txdata(1*32-1 downto 0*32);

-- Output relaxing
URt: PROCESS(clk)
BEGIN
IF (clk'event AND clk='1') THEN
	dCG_TxAv							<= CG_TxAv;
	--
	CG_TXData(64*(0+1)-1 downto 64*(0))	<= aCGTH_txdata(64*(1)-1-3 downto 64*(0)) & "001";
	CG_TXData(64*(1+1)-1 downto 64*(1))	<= aCGTH_txdata(64*(1)-1-3 downto 64*(0)) & "010"; -- XOR X"FFFFFFFFFFFFFFFF";
	CG_TXData(64*(2+1)-1 downto 64*(2))	<= aCGTH_txdata(64*(1)-1-3 downto 64*(0)) & "011"; -- XOR X"1122334455667788";
	CG_TXData(64*(3+1)-1 downto 64*(3))	<= aCGTH_txdata(64*(1)-1-3 downto 64*(0)) & "100"; -- XOR X"5566778899AABBCC";
	CG_TXData(64*(4+1)-1 downto 64*(4))	<= pck_cntr & aCGTH_txdata(64*(1)-32-1-3 downto 64*(0)) & "101"; -- XOR X"0123456789ABCDEF";
	--
	CG_TXSof							<= aCG_TXSof;
	CG_TXEof							<= aCG_TXEof;
	CG_TXDv							<= aCG_TXDv;
	CG_TXMod64						<= aCG_TXMod64;
END IF;
END PROCESS;

Ucfgx: aRippleRise PORT MAP( I=> Cfg_Wr, C=> clk, Q=> xCfg_Wr);

-- CG Trafficgen :)
UTXgen: PROCESS(clk, Rst, TX_State, ifs_cntr, dta_cntr)
BEGIN
IF (clk'event AND clk = '1') THEN

dRst				<= Rst;
dxCfg_Wr			<= xCfg_Wr;
dTxCfg_accepted	<= TxCfg_accepted;

IF (dxCfg_Wr = '1') THEN
	aCG_Mod40 			<= Cfg_Frlen(6-1 downto 0);
	aCG_Frlen 			<= Cfg_Frlen(16-1 downto 6);
	aCG_Ifslen			<= Cfg_Ifslen;
	TxCfg_wrwas			<= '1';
ELSIF (TxCfg_accepted = '1') THEN
	TxCfg_wrwas 			<= '0';
END IF;
--
IF (aCG_Ifslen = 0) THEN
	IFS_Zero		<= '1';
ELSE
	IFS_Zero		<= '0';
END IF;
--
IF (aCG_Mod40(6-1) = '1' OR aCG_Mod40(6-2 downto 2) = "111" OR aCG_Mod40(6-2 downto 0) = "00000") THEN -- > 27 OR 0
	IFS_Extend		<= '1';
ELSE
	IFS_Extend		<= '0';
END IF;
--
IF (TxCfg_accepted = '1') THEN
	CG_Mod40_reg			<= aCG_Mod40;
	CG_Frlen_reg			<= aCG_Frlen;
	IF (IFS_Zero = '1' AND IFS_Extend = '1' ) THEN -- Zero IFS not permitted! (+8 PRE +4 CRC +1EOF)
		CG_IFslen_reg			<= aCG_Ifslen(16-1 downto 1)&'1';
		IFS_Zero_4real			<= '0';
	ELSE	
		CG_IFslen_reg 			<= aCG_IFslen;
		IFS_Zero_4real			<= IFS_Zero;		
	END IF;
END IF;
--
INCR(aCGTH_txdata(32*(1)-1 downto 32*0));
--
IF (dta_cntr = CG_Frlen_reg) THEN
	PCK_Done		<= '1';
ELSE
	PCK_Done		<= '0';
END IF;
--
CASE TX_State IS
	WHEN TX_CONFIG =>
		aCG_TXSof			<= '0';
		aCG_TXEof			<= '0';
		aCG_TXDv			<= '0';
		aCG_TXMod64		<= "000000";
		IF (dRst = '1') THEN
			RESET(pck_cntr);
		END IF;
		TxCfg_accepted		<= TxCfg_wrwas;
		IF (dTxCfg_accepted = '1') THEN
			TX_State			<= TX_IFS;
		END IF;
	WHEN TX_IFS =>
		aCG_TXSof			<= '0';
		aCG_TXEof			<= '0';
		aCG_TXDv			<= '0';
		aCG_TXMod64		<= "000000";		
		dta_cntr			<= "0000000001";
		TxCfg_accepted		<= '0';
		INCR(ifs_cntr);
		IF (ifs_cntr = CG_Ifslen_reg) THEN
			TX_State		<= TX_SOF;
		END IF;
	WHEN TX_SOF =>
		IF (dRst = '1' OR TxCfg_wrwas = '1') THEN
			aCG_TXSof			<= '0';
			aCG_TXDv			<= '0';
			TX_State 			<= TX_CONFIG;
		ELSIF (dCG_TxAv = '1') THEN -- Clear to send
			aCG_TXSof			<= '1';
			aCG_TXDv			<= '1';
			TX_State 			<= TX_DATA;
		ELSE
			aCG_TXSof			<= '0';
			aCG_TXDv			<= '0';
		END IF;
		IF (dCG_TxAv = '1') THEN
			INCR(dta_cntr);
		END IF;
		aCG_TXEof			<= '0';
		aCG_TXMod64 		<= "000000";		
		ifs_cntr			<= X"0001";
	WHEN TX_DATA =>
		aCG_TXSof			<= '0';
		aCG_TXEof			<= PCK_Done;
		aCG_TXDv			<= '1';
		IF (PCK_Done = '1') THEN
			INCR(pck_cntr);
			dta_cntr			<= "0000000001";
			aCG_TXMod64		<= CG_Mod40_reg;
			IF (IFS_Zero_4real = '1') THEN
				TX_State 		<= TX_SOF;
			ELSE
				TX_State		<= TX_IFS;
			END IF;
		ELSE
			INCR(dta_cntr);
			aCG_TXMod64 		<= "000000";
		END IF;
	WHEN OTHERS =>
		NULL;
END CASE;
--
END IF;
END PROCESS; 

END CG_FrameGen_arch;
