--
-- Copyright (c) 2015 Aitia International Inc.
--
-- See the file "/doc/license.txt" for information on usage and redistribution
-- of this file, and for a DISCLAIMER OF ALL WARRANTIES.
-- 

LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_UNSIGNED.all;
USE work.tuw.all;

PACKAGE cgpcspma_v6 IS

COMPONENT ETH_100G_v6 IS
	PORT(
	-- GTH reference clocks (1 for every Quad)
	Q0_REFCLK_N				: IN  STD_LOGIC;
	Q0_REFCLK_P				: IN  STD_LOGIC;
	Q1_REFCLK_N				: IN  STD_LOGIC;
	Q1_REFCLK_P				: IN  STD_LOGIC;
	Q2_REFCLK_N				: IN  STD_LOGIC;
	Q2_REFCLK_P				: IN  STD_LOGIC;
	-- DRP clock
	DRP_CLK					: IN  STD_LOGIC;
--	CG_ExtClk					: IN  STD_LOGIC;
	-- GTH HS pins (only 10 pairs used @100G)
	RXN_GTH					: IN  STD_LOGIC_VECTOR(11 downto 0);
	RXP_GTH					: IN  STD_LOGIC_VECTOR(11 downto 0);
	TXN_GTH					: OUT STD_LOGIC_VECTOR(11 downto 0);
	TXP_GTH					: OUT STD_LOGIC_VECTOR(11 downto 0);
	-- 2 double rate clocks 4 MAC (312.5MHz)
	CG_Clk_RX					: OUT STD_LOGIC;
	CG_Clk_TX					: OUT STD_LOGIC;
	-- Rx MAC
	CG_RXData					: OUT STD_LOGIC_VECTOR(8*64-1 downto 0);
	CG_RXMod64				: OUT STD_LOGIC_VECTOR(6-1 downto 0);	
	CG_RXDv					: OUT STD_LOGIC;
	CG_RXSof					: OUT STD_LOGIC;
	CG_RXEof					: OUT STD_LOGIC;
	CG_RXCRCErr				: OUT STD_LOGIC;	
	CG_RXErr					: OUT STD_LOGIC;	
	-- Tx MAC
	CG_TXAv					: OUT STD_LOGIC;
	CG_TXData					: IN  STD_LOGIC_VECTOR(8*64-1 downto 0);
	CG_TXMod64				: IN  STD_LOGIC_VECTOR(6-1 downto 0);	
	CG_TXDv					: IN  STD_LOGIC;
	CG_TXSof					: IN  STD_LOGIC;
	CG_TXEof					: IN  STD_LOGIC;
	-- Etc
	PCS_RXSof 				: OUT STD_LOGIC;
	CG_Rst					: IN  STD_LOGIC;
	CG_CDR_Rst				: IN  STD_LOGIC;
	CG_signal_ok				: IN  STD_LOGIC;
	CG_initdone				: OUT STD_LOGIC;	
	CG_locked66b				: OUT STD_LOGIC;
	CG_aligned				: OUT STD_LOGIC;
	--
	DEBUG					: OUT STD_LOGIC_VECTOR(64-1 downto 0)
	);
END COMPONENT;

END PACKAGE;

library IEEE;
use IEEE.std_logic_1164.all;
USE IEEE.std_logic_UNSIGNED.ALL;
use work.tuw.all;
USE work.pcs_phy_100G_pkg.all;

library unisim;
use unisim.vcomponents.all;

ENTITY ETH_100G_v6 IS
	PORT(
	-- GTH reference clocks (1 for every Quad)
	Q0_REFCLK_N				: IN  STD_LOGIC;
	Q0_REFCLK_P				: IN  STD_LOGIC;
	Q1_REFCLK_N				: IN  STD_LOGIC;
	Q1_REFCLK_P				: IN  STD_LOGIC;
	Q2_REFCLK_N				: IN  STD_LOGIC;
	Q2_REFCLK_P				: IN  STD_LOGIC;
	-- DRP clock
	DRP_CLK					: IN  STD_LOGIC;
--	CG_ExtClk					: IN  STD_LOGIC;
	-- GTH HS pins (only 10 pairs used @100G)
	RXN_GTH					: IN  STD_LOGIC_VECTOR(11 downto 0);
	RXP_GTH					: IN  STD_LOGIC_VECTOR(11 downto 0);
	TXN_GTH					: OUT STD_LOGIC_VECTOR(11 downto 0);
	TXP_GTH					: OUT STD_LOGIC_VECTOR(11 downto 0);
	-- 2 double rate clocks 4 MAC (312.5MHz)
	CG_Clk_RX					: OUT STD_LOGIC;
	CG_Clk_TX					: OUT STD_LOGIC;
	-- Rx MAC
	CG_RXData					: OUT STD_LOGIC_VECTOR(8*64-1 downto 0);
	CG_RXMod64				: OUT STD_LOGIC_VECTOR(6-1 downto 0);
	CG_RXDv					: OUT STD_LOGIC;
	CG_RXSof					: OUT STD_LOGIC;
	CG_RXEof					: OUT STD_LOGIC;
	CG_RXCRCErr				: OUT STD_LOGIC;
	CG_RXErr					: OUT STD_LOGIC;
	-- Tx MAC
	CG_TXAv					: OUT STD_LOGIC;
	CG_TXData					: IN  STD_LOGIC_VECTOR(8*64-1 downto 0);
	CG_TXMod64				: IN  STD_LOGIC_VECTOR(6-1 downto 0);	
	CG_TXDv					: IN  STD_LOGIC;
	CG_TXSof					: IN  STD_LOGIC;
	CG_TXEof					: IN  STD_LOGIC;
	-- Etc
	PCS_RXSof 				: OUT STD_LOGIC;	
	CG_Rst					: IN  STD_LOGIC;
	CG_CDR_Rst				: IN  STD_LOGIC;
	CG_signal_ok				: IN  STD_LOGIC;
	CG_initdone				: OUT STD_LOGIC;
	CG_locked66b				: OUT STD_LOGIC;
	CG_aligned				: OUT STD_LOGIC;
	--
	DEBUG					: OUT STD_LOGIC_VECTOR(64-1 downto 0)
	);
END ETH_100G_v6;

ARCHITECTURE ETH_100G_v6_arch OF ETH_100G_v6 IS

COMPONENT CORE_100G is
  port (
    Clk_GTH_Rx : in STD_LOGIC; 
    Clk_GTH_Tx : in STD_LOGIC; 
    Clk_MAC_RX : in STD_LOGIC; 
    Clk_MAC_TX : in STD_LOGIC; 
    CG_TXDv : in STD_LOGIC; 
    CG_TXSof : in STD_LOGIC; 
    CG_TXEof : in STD_LOGIC; 
    CG_Rst : in STD_LOGIC; 
    CG_signal_ok : in STD_LOGIC; 
    CG_RXDv : out STD_LOGIC; 
    CG_RXSof : out STD_LOGIC; 
    CG_RXEof : out STD_LOGIC; 
    CG_RXCRCErr : out STD_LOGIC; 
    CG_RXErr : out STD_LOGIC; 
    CG_TXAv : out STD_LOGIC; 
    PCS_RXSof : out STD_LOGIC; 
    CG_locked66b : out STD_LOGIC; 
    CG_aligned : out STD_LOGIC; 
    DOUT_GTH : in STD_LOGIC_VECTOR ( 639 downto 0 ); 
    CG_TXData : in STD_LOGIC_VECTOR ( 511 downto 0 ); 
    CG_TXMod64 : in STD_LOGIC_VECTOR ( 5 downto 0 ); 
    DIN_GTH : out STD_LOGIC_VECTOR ( 639 downto 0 ); 
    CG_RXData : out STD_LOGIC_VECTOR ( 511 downto 0 ); 
    CG_RXMod64 : out STD_LOGIC_VECTOR ( 5 downto 0 ) 
  );
END COMPONENT;

SIGNAL DIN_GTH					: STD_LOGIC_VECTOR(10*64-1 downto 0);
SIGNAL DOUT_GTH				: STD_LOGIC_VECTOR(10*64-1 downto 0);
SIGNAL SLIP_GTH				: STD_LOGIC_VECTOR(10-1 downto 0);

SIGNAL clk_GTH_rx				: STD_LOGIC;
SIGNAL clk_GTH_tx				: STD_LOGIC;
SIGNAL clk_MAC_RX				: STD_LOGIC;
SIGNAL clk_MAC_TX				: STD_LOGIC;

SIGNAL dCDR_Rst 				: STD_LOGIC;
SIGNAL ddCDR_Rst 				: STD_LOGIC;
SIGNAL CDR_Rst_imp 				: STD_LOGIC;

attribute keep:string;

attribute keep of clk_MAC_RX :signal is "true";
attribute keep of clk_MAC_TX :signal is "true";
attribute keep of clk_GTH_rx :signal is "true";
attribute keep of clk_GTH_tx :signal is "true";

--SIGNAL drp_cntr				: STD_LOGIC_VECTOR(2-1 downto 0) := "00";
--SIGNAL DRP_CE					: STD_LOGIC := '0';
--SIGNAL DRP_CLK					: STD_LOGIC;

BEGIN

--
---- 0. MAC clock synthesis (312.5MHz)
--

CG_clk_RX	<= clk_MAC_RX;
CG_clk_TX	<= clk_MAC_TX;


--
---- 1. Physical GTH instantiation (3 GTH / 10(+2) lanes)
--
UGTH: pcs_phy_100G
	PORT MAP(
	-- GTH reference clocks (1 for every Quad)
	Q0_REFCLK_N				=> Q0_REFCLK_N, --: IN  STD_LOGIC;
	Q0_REFCLK_P				=> Q0_REFCLK_P, --: IN  STD_LOGIC;
	Q1_REFCLK_N				=> Q1_REFCLK_N, --: IN  STD_LOGIC;
	Q1_REFCLK_P				=> Q1_REFCLK_P, --: IN  STD_LOGIC;
	Q2_REFCLK_N				=> Q2_REFCLK_N, --: IN  STD_LOGIC;
	Q2_REFCLK_P				=> Q2_REFCLK_P, --: IN  STD_LOGIC;
--	-- DRP clock
	DRP_CLK					=> DRP_CLK, --: IN  STD_LOGIC;
	-- GTH HS pins (only 10 pairs used @100G)
	RXN_GTH					=> RXN_GTH, --: IN  STD_LOGIC_VECTOR(11 downto 0);
	RXP_GTH					=> RXP_GTH, --: IN  STD_LOGIC_VECTOR(11 downto 0);
	TXN_GTH					=> TXN_GTH, --: OUT STD_LOGIC_VECTOR(11 downto 0);
	TXP_GTH					=> TXP_GTH, --: OUT STD_LOGIC_VECTOR(11 downto 0)
	-- 1+1 recovered common PCS clocks (156.25MHz RX/TX)
	clk_GTH_rx				=> clk_GTH_rx, --: OUT STD_LOGIC;
	clk_GTH_tx				=> clk_GTH_tx, --: OUT STD_LOGIC;
	-- 2 double rate clocks 4 MAC (312.5MHz)
	clk_MAC_RX				=> clk_MAC_RX, --: OUT STD_LOGIC;
	clk_MAC_TX				=> clk_MAC_TX, --: OUT STD_LOGIC;
	-- RAW Data Output 4 every phys lane
	DOUT_GTH					=> DOUT_GTH, --: OUT STD_LOGIC_VECTOR(10*64-1 downto 0);
	-- RAW Data Input 4 every phys lane
	DIN_GTH					=> DIN_GTH, --: IN  STD_LOGIC_VECTOR(4*64-1 downto 0);
	-- Misc
	CDR_RST 					=> CDR_Rst_imp,
	GTH_INITDONE				=> CG_initdone
);

UCDR_Rstimp: PROCESS(clk_GTH_tx)
BEGIN
IF (clk_GTH_tx'event AND clk_GTH_tx='1') THEN
	dCDR_Rst		<= CG_CDR_Rst;
	ddCDR_Rst		<= dCDR_Rst;
	CDR_Rst_imp	<= dCDR_Rst AND (NOT ddCDR_Rst); -- Rise detector
END IF;
END PROCESS;

--
---- 2. 100G PCS/PMA + MAC
--
U100G: CORE_100G
	PORT MAP(
	-- 2 double rate clocks 4 MAC (312.5MHz)
	Clk_GTH_Rx 				=> clk_GTH_rx, --: IN  STD_LOGIC;
	Clk_GTH_Tx 				=> clk_GTH_tx, --: IN  STD_LOGIC;
	Clk_MAC_RX				=> clk_MAC_rx, --: IN  STD_LOGIC;
	Clk_MAC_TX				=> clk_MAC_tx, --: IN  STD_LOGIC;
	DIN_GTH					=> DIN_GTH, --: OUT STD_LOGIC_VECTOR(10*64-1 downto 0);
	DOUT_GTH					=> DOUT_GTH, --: IN  STD_LOGIC_VECTOR(10*64-1 downto 0);
	-- Rx MAC
	CG_RXData					=> CG_RXData, --: OUT STD_LOGIC_VECTOR(8*64-1 downto 0);
	CG_RXMod64				=> CG_RXMod64, --: OUT STD_LOGIC_VECTOR(6-1 downto 0);	
	CG_RXDv					=> CG_RXDv, --: OUT STD_LOGIC;
	CG_RXSof					=> CG_RXSof, --: OUT STD_LOGIC;
	CG_RXEof					=> CG_RXEof, --: OUT STD_LOGIC;
	CG_RXCRCErr				=> CG_RXCRCErr, --: OUT STD_LOGIC;	
	CG_RXErr					=> CG_RXErr, --: OUT STD_LOGIC;	
	-- Tx MAC
	CG_TXAv					=> CG_TXAv, --: OUT STD_LOGIC;
	CG_TXData					=> CG_TXData, --: IN  STD_LOGIC_VECTOR(8*64-1 downto 0);
	CG_TXMod64				=> CG_TXMod64, --: IN  STD_LOGIC_VECTOR(6-1 downto 0);	
	CG_TXDv					=> CG_TXDv, --: IN  STD_LOGIC;
	CG_TXSof					=> CG_TXSof, --: IN  STD_LOGIC;
	CG_TXEof					=> CG_TXEof, --: IN  STD_LOGIC;
	-- Etc
	PCS_RXSof 				=> PCS_RXSof, --: OUT STD_LOGIC;
	CG_Rst					=> CG_Rst, --: IN  STD_LOGIC;
	CG_signal_ok				=> CG_signal_ok, --: IN  STD_LOGIC;
	CG_locked66b				=> CG_locked66b, --: OUT STD_LOGIC;
	CG_aligned				=> CG_aligned --: OUT STD_LOGIC
	);

END ETH_100G_v6_arch;
