--
-- Copyright (c) 2015 Aitia International Inc.
--
-- See the file "/doc/license.txt" for information on usage and redistribution
-- of this file, and for a DISCLAIMER OF ALL WARRANTIES.
-- 

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
--USE work.pcs_marker_100G_pkg.all;

PACKAGE pcs_phy_100G_pkg IS

COMPONENT pcs_phy_100G IS
	PORT(
	-- GTH reference clocks (1 for every Quad)
	Q0_REFCLK_N				: IN  STD_LOGIC;
	Q0_REFCLK_P				: IN  STD_LOGIC;
	Q1_REFCLK_N				: IN  STD_LOGIC;
	Q1_REFCLK_P				: IN  STD_LOGIC;
	Q2_REFCLK_N				: IN  STD_LOGIC;
	Q2_REFCLK_P				: IN  STD_LOGIC;
--	-- DRP clock
	DRP_CLK					: IN  STD_LOGIC;
	-- GTH HS pins (only 10 pairs used @100G)
	RXN_GTH					: IN  STD_LOGIC_VECTOR(11 downto 0);
	RXP_GTH					: IN  STD_LOGIC_VECTOR(11 downto 0);
	TXN_GTH					: OUT STD_LOGIC_VECTOR(11 downto 0);
	TXP_GTH					: OUT STD_LOGIC_VECTOR(11 downto 0);
	-- 1+1 recovered common PCS clocks (161.133MHz RX/TX)
	clk_GTH_rx				: OUT STD_LOGIC;
	clk_GTH_tx				: OUT STD_LOGIC;
	-- 2 double rate clocks 4 MAC (312.5MHz)
	clk_MAC_RX				: OUT STD_LOGIC;
	clk_MAC_TX				: OUT STD_LOGIC;
	-- RAW Data Output 4 every phys lane
	DOUT_GTH					: OUT STD_LOGIC_VECTOR(10*64-1 downto 0);
	-- RAW Data Input 4 every phys lane
	DIN_GTH					: IN  STD_LOGIC_VECTOR(10*64-1 downto 0);
	-- Misc
	CDR_RST					: IN  STD_LOGIC;	
	GTH_INITDONE				: OUT STD_LOGIC
--	SLIP_GTH					: IN  STD_LOGIC_VECTOR(PCS_PHY_LANES-1 downto 0)
);
END COMPONENT;

END PACKAGE;


library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
--USE work.pcs_marker_100G_pkg.all;

library UNISIM;
use UNISIM.VCOMPONENTS.ALL;


ENTITY pcs_phy_100G IS
	PORT(
	-- GTH reference clocks (1 for every Quad)
	Q0_REFCLK_N				: IN  STD_LOGIC;
	Q0_REFCLK_P				: IN  STD_LOGIC;
	Q1_REFCLK_N				: IN  STD_LOGIC;
	Q1_REFCLK_P				: IN  STD_LOGIC;
	Q2_REFCLK_N				: IN  STD_LOGIC;
	Q2_REFCLK_P				: IN  STD_LOGIC;
--	-- DRP clock
	DRP_CLK					: IN  STD_LOGIC;
	-- GTH HS pins (only 10 pairs used @100G)
	RXN_GTH					: IN  STD_LOGIC_VECTOR(11 downto 0);
	RXP_GTH					: IN  STD_LOGIC_VECTOR(11 downto 0);
	TXN_GTH					: OUT STD_LOGIC_VECTOR(11 downto 0);
	TXP_GTH					: OUT STD_LOGIC_VECTOR(11 downto 0);
	-- 1+1 recovered common PCS clocks (166.133MHz RX/TX)
	clk_GTH_rx				: OUT STD_LOGIC;
	clk_GTH_tx				: OUT STD_LOGIC;
	-- 2 double rate clocks 4 MAC (312.5MHz)
	clk_MAC_RX				: OUT STD_LOGIC;
	clk_MAC_TX				: OUT STD_LOGIC;
	-- RAW Data Output 4 every phys lane
	DOUT_GTH					: OUT STD_LOGIC_VECTOR(10*64-1 downto 0);
	-- RAW Data Input 4 every phys lane
	DIN_GTH					: IN  STD_LOGIC_VECTOR(10*64-1 downto 0);
	-- Misc
	CDR_RST					: IN  STD_LOGIC;
	GTH_INITDONE				: OUT STD_LOGIC	
--	SLIP_GTH					: IN  STD_LOGIC_VECTOR(PCS_PHY_LANES-1 downto 0)
);
END pcs_phy_100G;
    
ARCHITECTURE pcs_phy_100G_arch OF pcs_phy_100G IS

component v6_gthwizard_v1_11_caui
generic
(
    WRAPPER_SIM_GTHRESET_SPEEDUP : integer   := 0      -- Set to 1 to speed up sim reset
);
port
(
    
    --_________________________________________________________________________
    --_________________________________________________________________________
    --QUAD0  (X1Y0)

    --------------------------------- Resetdone --------------------------------
    QUAD0_TX_PCS_RESETDONE0_OUT        : out  std_logic;
    QUAD0_RX_PCS_CDR_RESETDONE0_OUT    : out  std_logic;
    QUAD0_TX_PCS_RESETDONE1_OUT        : out  std_logic;
    QUAD0_RX_PCS_CDR_RESETDONE1_OUT    : out  std_logic;
    QUAD0_TX_PCS_RESETDONE2_OUT        : out  std_logic;
    QUAD0_RX_PCS_CDR_RESETDONE2_OUT    : out  std_logic;
    QUAD0_TX_PCS_RESETDONE3_OUT        : out  std_logic;
    QUAD0_RX_PCS_CDR_RESETDONE3_OUT    : out  std_logic;
    --------------------------------- Initdone ---------------------------------
    QUAD0_GTHINITDONE_OUT                       : out  std_logic;    
    --------------------------------- PCS Resets -------------------------------
    QUAD0_TX_PCS_RESET0_IN             : in  std_logic;
    QUAD0_RX_PCS_CDR_RESET0_IN         : in  std_logic;
    QUAD0_TX_PCS_RESET1_IN             : in  std_logic;
    QUAD0_RX_PCS_CDR_RESET1_IN         : in  std_logic;
    QUAD0_TX_PCS_RESET2_IN             : in  std_logic;
    QUAD0_RX_PCS_CDR_RESET2_IN         : in  std_logic;
    QUAD0_TX_PCS_RESET3_IN             : in  std_logic;
    QUAD0_RX_PCS_CDR_RESET3_IN         : in  std_logic;
    --------------------------------- Powerdown --------------------------------
    QUAD0_POWERDOWN0_IN                     : in   std_logic;
    QUAD0_POWERDOWN1_IN                     : in   std_logic;
    QUAD0_POWERDOWN2_IN                     : in   std_logic;
    QUAD0_POWERDOWN3_IN                     : in   std_logic;
    QUAD0_RXPOWERDOWN0_IN                   : in   std_logic_vector(1 downto 0);
    QUAD0_RXPOWERDOWN1_IN                   : in   std_logic_vector(1 downto 0);
    QUAD0_RXPOWERDOWN2_IN                   : in   std_logic_vector(1 downto 0);
    QUAD0_RXPOWERDOWN3_IN                   : in   std_logic_vector(1 downto 0);
    QUAD0_TXPOWERDOWN0_IN                   : in   std_logic_vector(1 downto 0);
    QUAD0_TXPOWERDOWN1_IN                   : in   std_logic_vector(1 downto 0);
    QUAD0_TXPOWERDOWN2_IN                   : in   std_logic_vector(1 downto 0);
    QUAD0_TXPOWERDOWN3_IN                   : in   std_logic_vector(1 downto 0);
    ------------------------------- Receive Ports ------------------------------
    QUAD0_RXBUFRESET0_IN                    : in   std_logic;
    QUAD0_RXBUFRESET1_IN                    : in   std_logic;
    QUAD0_RXBUFRESET2_IN                    : in   std_logic;
    QUAD0_RXBUFRESET3_IN                    : in   std_logic;
    QUAD0_RXCTRLACK0_OUT                    : out  std_logic;
    QUAD0_RXCTRLACK1_OUT                    : out  std_logic;
    QUAD0_RXCTRLACK2_OUT                    : out  std_logic;
    QUAD0_RXCTRLACK3_OUT                    : out  std_logic;
    QUAD0_RXDATA0_OUT                       : out  std_logic_vector(63 downto 0);
    QUAD0_RXDATA1_OUT                       : out  std_logic_vector(63 downto 0);
    QUAD0_RXDATA2_OUT                       : out  std_logic_vector(63 downto 0);
    QUAD0_RXDATA3_OUT                       : out  std_logic_vector(63 downto 0);
    QUAD0_RXN0_IN                           : in   std_logic;
    QUAD0_RXN1_IN                           : in   std_logic;
    QUAD0_RXN2_IN                           : in   std_logic;
    QUAD0_RXN3_IN                           : in   std_logic;
    QUAD0_RXP0_IN                           : in   std_logic;
    QUAD0_RXP1_IN                           : in   std_logic;
    QUAD0_RXP2_IN                           : in   std_logic;
    QUAD0_RXP3_IN                           : in   std_logic;
    QUAD0_RXSLIP0_IN                        : in   std_logic;
    QUAD0_RXSLIP1_IN                        : in   std_logic;
    QUAD0_RXSLIP2_IN                        : in   std_logic;
    QUAD0_RXSLIP3_IN                        : in   std_logic;
    QUAD0_RXUSERCLKIN0_IN                   : in   std_logic;
    QUAD0_RXUSERCLKIN1_IN                   : in   std_logic;
    QUAD0_RXUSERCLKIN2_IN                   : in   std_logic;
    QUAD0_RXUSERCLKIN3_IN                   : in   std_logic;
    QUAD0_RXUSERCLKOUT0_OUT                 : out  std_logic;
    QUAD0_RXUSERCLKOUT1_OUT                 : out  std_logic;
    QUAD0_RXUSERCLKOUT2_OUT                 : out  std_logic;
    QUAD0_RXUSERCLKOUT3_OUT                 : out  std_logic;
    ------------- Shared Ports - Dynamic Reconfiguration Port (DRP) ------------
    QUAD0_DADDR_IN                          : in   std_logic_vector(15 downto 0);
    QUAD0_DCLK_IN                           : in   std_logic;
    QUAD0_DEN_IN                            : in   std_logic;
    QUAD0_DI_IN                             : in   std_logic_vector(15 downto 0);
    QUAD0_DISABLEDRP_IN                     : in   std_logic;
    QUAD0_DRDY_OUT                          : out  std_logic;
    QUAD0_DRPDO_OUT                         : out  std_logic_vector(15 downto 0);
    QUAD0_DWE_IN                            : in   std_logic;
    ---------------------------- Shared Ports - Other --------------------------
    QUAD0_GTHINIT_IN                        : in   std_logic;
    QUAD0_GTHRESET_IN                       : in   std_logic;
    QUAD0_MGMTPCSLANESEL_IN                 : in   std_logic_vector(3 downto 0);
    QUAD0_MGMTPCSMMDADDR_IN                 : in   std_logic_vector(4 downto 0);
    QUAD0_MGMTPCSRDACK_OUT                  : out  std_logic;
    QUAD0_MGMTPCSRDDATA_OUT                 : out  std_logic_vector(15 downto 0);
    QUAD0_MGMTPCSREGADDR_IN                 : in   std_logic_vector(15 downto 0);
    QUAD0_MGMTPCSREGRD_IN                   : in   std_logic;
    QUAD0_MGMTPCSREGWR_IN                   : in   std_logic;
    QUAD0_MGMTPCSWRDATA_IN                  : in   std_logic_vector(15 downto 0);
    QUAD0_REFCLK_IN                         : in   std_logic;
    QUAD0_REFCLK_OUT                        : OUT  STD_LOGIC;    
    ------------------------------- Transmit Ports -----------------------------
    QUAD0_TXBUFRESET0_IN                    : in   std_logic;
    QUAD0_TXBUFRESET1_IN                    : in   std_logic;
    QUAD0_TXBUFRESET2_IN                    : in   std_logic;
    QUAD0_TXBUFRESET3_IN                    : in   std_logic;
    QUAD0_TXCTRLACK0_OUT                    : out  std_logic;
    QUAD0_TXCTRLACK1_OUT                    : out  std_logic;
    QUAD0_TXCTRLACK2_OUT                    : out  std_logic;
    QUAD0_TXCTRLACK3_OUT                    : out  std_logic;
    QUAD0_TXDATA0_IN                        : in   std_logic_vector(63 downto 0);
    QUAD0_TXDATA1_IN                        : in   std_logic_vector(63 downto 0);
    QUAD0_TXDATA2_IN                        : in   std_logic_vector(63 downto 0);
    QUAD0_TXDATA3_IN                        : in   std_logic_vector(63 downto 0);
    QUAD0_TXN0_OUT                          : out  std_logic;
    QUAD0_TXN1_OUT                          : out  std_logic;
    QUAD0_TXN2_OUT                          : out  std_logic;
    QUAD0_TXN3_OUT                          : out  std_logic;
    QUAD0_TXP0_OUT                          : out  std_logic;
    QUAD0_TXP1_OUT                          : out  std_logic;
    QUAD0_TXP2_OUT                          : out  std_logic;
    QUAD0_TXP3_OUT                          : out  std_logic;
    QUAD0_TXUSERCLKIN0_IN                   : in   std_logic;
    QUAD0_TXUSERCLKIN1_IN                   : in   std_logic;
    QUAD0_TXUSERCLKIN2_IN                   : in   std_logic;
    QUAD0_TXUSERCLKIN3_IN                   : in   std_logic;
    QUAD0_TXUSERCLKOUT0_OUT                 : out  std_logic;
    QUAD0_TXUSERCLKOUT1_OUT                 : out  std_logic;
    QUAD0_TXUSERCLKOUT2_OUT                 : out  std_logic;
    QUAD0_TXUSERCLKOUT3_OUT                 : out  std_logic;


    
    --_________________________________________________________________________
    --_________________________________________________________________________
    --QUAD1  (X1Y1)

    --------------------------------- Resetdone --------------------------------
    QUAD1_TX_PCS_RESETDONE0_OUT        : out  std_logic;
    QUAD1_RX_PCS_CDR_RESETDONE0_OUT    : out  std_logic;
    QUAD1_TX_PCS_RESETDONE1_OUT        : out  std_logic;
    QUAD1_RX_PCS_CDR_RESETDONE1_OUT    : out  std_logic;
    QUAD1_TX_PCS_RESETDONE2_OUT        : out  std_logic;
    QUAD1_RX_PCS_CDR_RESETDONE2_OUT    : out  std_logic;
    QUAD1_TX_PCS_RESETDONE3_OUT        : out  std_logic;
    QUAD1_RX_PCS_CDR_RESETDONE3_OUT    : out  std_logic;
    --------------------------------- Initdone ---------------------------------
    QUAD1_GTHINITDONE_OUT                       : out  std_logic;    
    --------------------------------- PCS Resets -------------------------------
    QUAD1_TX_PCS_RESET0_IN             : in  std_logic;
    QUAD1_RX_PCS_CDR_RESET0_IN         : in  std_logic;
    QUAD1_TX_PCS_RESET1_IN             : in  std_logic;
    QUAD1_RX_PCS_CDR_RESET1_IN         : in  std_logic;
    QUAD1_TX_PCS_RESET2_IN             : in  std_logic;
    QUAD1_RX_PCS_CDR_RESET2_IN         : in  std_logic;
    QUAD1_TX_PCS_RESET3_IN             : in  std_logic;
    QUAD1_RX_PCS_CDR_RESET3_IN         : in  std_logic;
    --------------------------------- Powerdown --------------------------------
    QUAD1_POWERDOWN0_IN                     : in   std_logic;
    QUAD1_POWERDOWN1_IN                     : in   std_logic;
    QUAD1_POWERDOWN2_IN                     : in   std_logic;
    QUAD1_POWERDOWN3_IN                     : in   std_logic;
    QUAD1_RXPOWERDOWN0_IN                   : in   std_logic_vector(1 downto 0);
    QUAD1_RXPOWERDOWN1_IN                   : in   std_logic_vector(1 downto 0);
    QUAD1_RXPOWERDOWN2_IN                   : in   std_logic_vector(1 downto 0);
    QUAD1_RXPOWERDOWN3_IN                   : in   std_logic_vector(1 downto 0);
    QUAD1_TXPOWERDOWN0_IN                   : in   std_logic_vector(1 downto 0);
    QUAD1_TXPOWERDOWN1_IN                   : in   std_logic_vector(1 downto 0);
    QUAD1_TXPOWERDOWN2_IN                   : in   std_logic_vector(1 downto 0);
    QUAD1_TXPOWERDOWN3_IN                   : in   std_logic_vector(1 downto 0);
    ------------------------------- Receive Ports ------------------------------
    QUAD1_RXBUFRESET0_IN                    : in   std_logic;
    QUAD1_RXBUFRESET1_IN                    : in   std_logic;
    QUAD1_RXBUFRESET2_IN                    : in   std_logic;
    QUAD1_RXBUFRESET3_IN                    : in   std_logic;
    QUAD1_RXCTRLACK0_OUT                    : out  std_logic;
    QUAD1_RXCTRLACK1_OUT                    : out  std_logic;
    QUAD1_RXCTRLACK2_OUT                    : out  std_logic;
    QUAD1_RXCTRLACK3_OUT                    : out  std_logic;
    QUAD1_RXDATA0_OUT                       : out  std_logic_vector(63 downto 0);
    QUAD1_RXDATA1_OUT                       : out  std_logic_vector(63 downto 0);
    QUAD1_RXDATA2_OUT                       : out  std_logic_vector(63 downto 0);
    QUAD1_RXDATA3_OUT                       : out  std_logic_vector(63 downto 0);
    QUAD1_RXN0_IN                           : in   std_logic;
    QUAD1_RXN1_IN                           : in   std_logic;
    QUAD1_RXN2_IN                           : in   std_logic;
    QUAD1_RXN3_IN                           : in   std_logic;
    QUAD1_RXP0_IN                           : in   std_logic;
    QUAD1_RXP1_IN                           : in   std_logic;
    QUAD1_RXP2_IN                           : in   std_logic;
    QUAD1_RXP3_IN                           : in   std_logic;
    QUAD1_RXSLIP0_IN                        : in   std_logic;
    QUAD1_RXSLIP1_IN                        : in   std_logic;
    QUAD1_RXSLIP2_IN                        : in   std_logic;
    QUAD1_RXSLIP3_IN                        : in   std_logic;
    QUAD1_RXUSERCLKIN0_IN                   : in   std_logic;
    QUAD1_RXUSERCLKIN1_IN                   : in   std_logic;
    QUAD1_RXUSERCLKIN2_IN                   : in   std_logic;
    QUAD1_RXUSERCLKIN3_IN                   : in   std_logic;
    QUAD1_RXUSERCLKOUT0_OUT                 : out  std_logic;
    QUAD1_RXUSERCLKOUT1_OUT                 : out  std_logic;
    QUAD1_RXUSERCLKOUT2_OUT                 : out  std_logic;
    QUAD1_RXUSERCLKOUT3_OUT                 : out  std_logic;
    ------------- Shared Ports - Dynamic Reconfiguration Port (DRP) ------------
    QUAD1_DADDR_IN                          : in   std_logic_vector(15 downto 0);
    QUAD1_DCLK_IN                           : in   std_logic;
    QUAD1_DEN_IN                            : in   std_logic;
    QUAD1_DI_IN                             : in   std_logic_vector(15 downto 0);
    QUAD1_DISABLEDRP_IN                     : in   std_logic;
    QUAD1_DRDY_OUT                          : out  std_logic;
    QUAD1_DRPDO_OUT                         : out  std_logic_vector(15 downto 0);
    QUAD1_DWE_IN                            : in   std_logic;
    ---------------------------- Shared Ports - Other --------------------------
    QUAD1_GTHINIT_IN                        : in   std_logic;
    QUAD1_GTHRESET_IN                       : in   std_logic;
    QUAD1_MGMTPCSLANESEL_IN                 : in   std_logic_vector(3 downto 0);
    QUAD1_MGMTPCSMMDADDR_IN                 : in   std_logic_vector(4 downto 0);
    QUAD1_MGMTPCSRDACK_OUT                  : out  std_logic;
    QUAD1_MGMTPCSRDDATA_OUT                 : out  std_logic_vector(15 downto 0);
    QUAD1_MGMTPCSREGADDR_IN                 : in   std_logic_vector(15 downto 0);
    QUAD1_MGMTPCSREGRD_IN                   : in   std_logic;
    QUAD1_MGMTPCSREGWR_IN                   : in   std_logic;
    QUAD1_MGMTPCSWRDATA_IN                  : in   std_logic_vector(15 downto 0);
    QUAD1_REFCLK_IN                         : in   std_logic;
    ------------------------------- Transmit Ports -----------------------------
    QUAD1_TXBUFRESET0_IN                    : in   std_logic;
    QUAD1_TXBUFRESET1_IN                    : in   std_logic;
    QUAD1_TXBUFRESET2_IN                    : in   std_logic;
    QUAD1_TXBUFRESET3_IN                    : in   std_logic;
    QUAD1_TXCTRLACK0_OUT                    : out  std_logic;
    QUAD1_TXCTRLACK1_OUT                    : out  std_logic;
    QUAD1_TXCTRLACK2_OUT                    : out  std_logic;
    QUAD1_TXCTRLACK3_OUT                    : out  std_logic;
    QUAD1_TXDATA0_IN                        : in   std_logic_vector(63 downto 0);
    QUAD1_TXDATA1_IN                        : in   std_logic_vector(63 downto 0);
    QUAD1_TXDATA2_IN                        : in   std_logic_vector(63 downto 0);
    QUAD1_TXDATA3_IN                        : in   std_logic_vector(63 downto 0);
    QUAD1_TXN0_OUT                          : out  std_logic;
    QUAD1_TXN1_OUT                          : out  std_logic;
    QUAD1_TXN2_OUT                          : out  std_logic;
    QUAD1_TXN3_OUT                          : out  std_logic;
    QUAD1_TXP0_OUT                          : out  std_logic;
    QUAD1_TXP1_OUT                          : out  std_logic;
    QUAD1_TXP2_OUT                          : out  std_logic;
    QUAD1_TXP3_OUT                          : out  std_logic;
    QUAD1_TXUSERCLKIN0_IN                   : in   std_logic;
    QUAD1_TXUSERCLKIN1_IN                   : in   std_logic;
    QUAD1_TXUSERCLKIN2_IN                   : in   std_logic;
    QUAD1_TXUSERCLKIN3_IN                   : in   std_logic;
    QUAD1_TXUSERCLKOUT0_OUT                 : out  std_logic;
    QUAD1_TXUSERCLKOUT1_OUT                 : out  std_logic;
    QUAD1_TXUSERCLKOUT2_OUT                 : out  std_logic;
    QUAD1_TXUSERCLKOUT3_OUT                 : out  std_logic;


    
    --_________________________________________________________________________
    --_________________________________________________________________________
    --QUAD2  (X1Y2)

    --------------------------------- Resetdone --------------------------------
    QUAD2_TX_PCS_RESETDONE0_OUT        : out  std_logic;
    QUAD2_RX_PCS_CDR_RESETDONE0_OUT    : out  std_logic;
    QUAD2_TX_PCS_RESETDONE1_OUT        : out  std_logic;
    QUAD2_RX_PCS_CDR_RESETDONE1_OUT    : out  std_logic;
    QUAD2_TX_PCS_RESETDONE2_OUT        : out  std_logic;
    QUAD2_RX_PCS_CDR_RESETDONE2_OUT    : out  std_logic;
    QUAD2_TX_PCS_RESETDONE3_OUT        : out  std_logic;
    QUAD2_RX_PCS_CDR_RESETDONE3_OUT    : out  std_logic;
    --------------------------------- Initdone ---------------------------------
    QUAD2_GTHINITDONE_OUT                       : out  std_logic;    
    --------------------------------- PCS Resets -------------------------------
    QUAD2_TX_PCS_RESET0_IN             : in  std_logic;
    QUAD2_RX_PCS_CDR_RESET0_IN         : in  std_logic;
    QUAD2_TX_PCS_RESET1_IN             : in  std_logic;
    QUAD2_RX_PCS_CDR_RESET1_IN         : in  std_logic;
    QUAD2_TX_PCS_RESET2_IN             : in  std_logic;
    QUAD2_RX_PCS_CDR_RESET2_IN         : in  std_logic;
    QUAD2_TX_PCS_RESET3_IN             : in  std_logic;
    QUAD2_RX_PCS_CDR_RESET3_IN         : in  std_logic;
    --------------------------------- Powerdown --------------------------------
    QUAD2_POWERDOWN0_IN                     : in   std_logic;
    QUAD2_POWERDOWN1_IN                     : in   std_logic;
    QUAD2_POWERDOWN2_IN                     : in   std_logic;
    QUAD2_POWERDOWN3_IN                     : in   std_logic;
    QUAD2_RXPOWERDOWN0_IN                   : in   std_logic_vector(1 downto 0);
    QUAD2_RXPOWERDOWN1_IN                   : in   std_logic_vector(1 downto 0);
    QUAD2_RXPOWERDOWN2_IN                   : in   std_logic_vector(1 downto 0);
    QUAD2_RXPOWERDOWN3_IN                   : in   std_logic_vector(1 downto 0);
    QUAD2_TXPOWERDOWN0_IN                   : in   std_logic_vector(1 downto 0);
    QUAD2_TXPOWERDOWN1_IN                   : in   std_logic_vector(1 downto 0);
    QUAD2_TXPOWERDOWN2_IN                   : in   std_logic_vector(1 downto 0);
    QUAD2_TXPOWERDOWN3_IN                   : in   std_logic_vector(1 downto 0);
    ------------------------------- Receive Ports ------------------------------
    QUAD2_RXBUFRESET0_IN                    : in   std_logic;
    QUAD2_RXBUFRESET1_IN                    : in   std_logic;
    QUAD2_RXBUFRESET2_IN                    : in   std_logic;
    QUAD2_RXBUFRESET3_IN                    : in   std_logic;
    QUAD2_RXCTRLACK0_OUT                    : out  std_logic;
    QUAD2_RXCTRLACK1_OUT                    : out  std_logic;
    QUAD2_RXCTRLACK2_OUT                    : out  std_logic;
    QUAD2_RXCTRLACK3_OUT                    : out  std_logic;
    QUAD2_RXDATA0_OUT                       : out  std_logic_vector(63 downto 0);
    QUAD2_RXDATA1_OUT                       : out  std_logic_vector(63 downto 0);
    QUAD2_RXDATA2_OUT                       : out  std_logic_vector(63 downto 0);
    QUAD2_RXDATA3_OUT                       : out  std_logic_vector(63 downto 0);
    QUAD2_RXN0_IN                           : in   std_logic;
    QUAD2_RXN1_IN                           : in   std_logic;
    QUAD2_RXN2_IN                           : in   std_logic;
    QUAD2_RXN3_IN                           : in   std_logic;
    QUAD2_RXP0_IN                           : in   std_logic;
    QUAD2_RXP1_IN                           : in   std_logic;
    QUAD2_RXP2_IN                           : in   std_logic;
    QUAD2_RXP3_IN                           : in   std_logic;
    QUAD2_RXSLIP0_IN                        : in   std_logic;
    QUAD2_RXSLIP1_IN                        : in   std_logic;
    QUAD2_RXSLIP2_IN                        : in   std_logic;
    QUAD2_RXSLIP3_IN                        : in   std_logic;
    QUAD2_RXUSERCLKIN0_IN                   : in   std_logic;
    QUAD2_RXUSERCLKIN1_IN                   : in   std_logic;
    QUAD2_RXUSERCLKIN2_IN                   : in   std_logic;
    QUAD2_RXUSERCLKIN3_IN                   : in   std_logic;
    QUAD2_RXUSERCLKOUT0_OUT                 : out  std_logic;
    QUAD2_RXUSERCLKOUT1_OUT                 : out  std_logic;
    QUAD2_RXUSERCLKOUT2_OUT                 : out  std_logic;
    QUAD2_RXUSERCLKOUT3_OUT                 : out  std_logic;
    ------------- Shared Ports - Dynamic Reconfiguration Port (DRP) ------------
    QUAD2_DADDR_IN                          : in   std_logic_vector(15 downto 0);
    QUAD2_DCLK_IN                           : in   std_logic;
    QUAD2_DEN_IN                            : in   std_logic;
    QUAD2_DI_IN                             : in   std_logic_vector(15 downto 0);
    QUAD2_DISABLEDRP_IN                     : in   std_logic;
    QUAD2_DRDY_OUT                          : out  std_logic;
    QUAD2_DRPDO_OUT                         : out  std_logic_vector(15 downto 0);
    QUAD2_DWE_IN                            : in   std_logic;
    ---------------------------- Shared Ports - Other --------------------------
    QUAD2_GTHINIT_IN                        : in   std_logic;
    QUAD2_GTHRESET_IN                       : in   std_logic;
    QUAD2_MGMTPCSLANESEL_IN                 : in   std_logic_vector(3 downto 0);
    QUAD2_MGMTPCSMMDADDR_IN                 : in   std_logic_vector(4 downto 0);
    QUAD2_MGMTPCSRDACK_OUT                  : out  std_logic;
    QUAD2_MGMTPCSRDDATA_OUT                 : out  std_logic_vector(15 downto 0);
    QUAD2_MGMTPCSREGADDR_IN                 : in   std_logic_vector(15 downto 0);
    QUAD2_MGMTPCSREGRD_IN                   : in   std_logic;
    QUAD2_MGMTPCSREGWR_IN                   : in   std_logic;
    QUAD2_MGMTPCSWRDATA_IN                  : in   std_logic_vector(15 downto 0);
    QUAD2_REFCLK_IN                         : in   std_logic;
    ------------------------------- Transmit Ports -----------------------------
    QUAD2_TXBUFRESET0_IN                    : in   std_logic;
    QUAD2_TXBUFRESET1_IN                    : in   std_logic;
    QUAD2_TXBUFRESET2_IN                    : in   std_logic;
    QUAD2_TXBUFRESET3_IN                    : in   std_logic;
    QUAD2_TXCTRLACK0_OUT                    : out  std_logic;
    QUAD2_TXCTRLACK1_OUT                    : out  std_logic;
    QUAD2_TXCTRLACK2_OUT                    : out  std_logic;
    QUAD2_TXCTRLACK3_OUT                    : out  std_logic;
    QUAD2_TXDATA0_IN                        : in   std_logic_vector(63 downto 0);
    QUAD2_TXDATA1_IN                        : in   std_logic_vector(63 downto 0);
    QUAD2_TXDATA2_IN                        : in   std_logic_vector(63 downto 0);
    QUAD2_TXDATA3_IN                        : in   std_logic_vector(63 downto 0);
    QUAD2_TXN0_OUT                          : out  std_logic;
    QUAD2_TXN1_OUT                          : out  std_logic;
    QUAD2_TXN2_OUT                          : out  std_logic;
    QUAD2_TXN3_OUT                          : out  std_logic;
    QUAD2_TXP0_OUT                          : out  std_logic;
    QUAD2_TXP1_OUT                          : out  std_logic;
    QUAD2_TXP2_OUT                          : out  std_logic;
    QUAD2_TXP3_OUT                          : out  std_logic;
    QUAD2_TXUSERCLKIN0_IN                   : in   std_logic;
    QUAD2_TXUSERCLKIN1_IN                   : in   std_logic;
    QUAD2_TXUSERCLKIN2_IN                   : in   std_logic;
    QUAD2_TXUSERCLKIN3_IN                   : in   std_logic;
    QUAD2_TXUSERCLKOUT0_OUT                 : out  std_logic;
    QUAD2_TXUSERCLKOUT1_OUT                 : out  std_logic;
    QUAD2_TXUSERCLKOUT2_OUT                 : out  std_logic;
    QUAD2_TXUSERCLKOUT3_OUT                 : out  std_logic


);
end component;

--***********************************Parameter Declarations********************

    constant DLY : time := 1 ns;

--**************************** Wire Declarations ******************************
    -------------------------- MGT Wrapper Wires ------------------------------
    --________________________________________________________________________
    --________________________________________________________________________
    --QUAD0   (X1Y0)

    --------------------------------- Powerdown --------------------------------
    signal  quad0_powerdown0_i              : std_logic;
    signal  quad0_powerdown1_i              : std_logic;
    signal  quad0_powerdown2_i              : std_logic;
    signal  quad0_powerdown3_i              : std_logic;
    signal  quad0_rxpowerdown0_i            : std_logic_vector(1 downto 0);
    signal  quad0_rxpowerdown1_i            : std_logic_vector(1 downto 0);
    signal  quad0_rxpowerdown2_i            : std_logic_vector(1 downto 0);
    signal  quad0_rxpowerdown3_i            : std_logic_vector(1 downto 0);
    signal  quad0_txpowerdown0_i            : std_logic_vector(1 downto 0);
    signal  quad0_txpowerdown1_i            : std_logic_vector(1 downto 0);
    signal  quad0_txpowerdown2_i            : std_logic_vector(1 downto 0);
    signal  quad0_txpowerdown3_i            : std_logic_vector(1 downto 0);
    ------------------------------- Receive Ports ------------------------------
    signal  quad0_rxbufreset0_i             : std_logic;
    signal  quad0_rxbufreset1_i             : std_logic;
    signal  quad0_rxbufreset2_i             : std_logic;
    signal  quad0_rxbufreset3_i             : std_logic;
    signal  quad0_rxctrlack0_i              : std_logic;
    signal  quad0_rxctrlack1_i              : std_logic;
    signal  quad0_rxctrlack2_i              : std_logic;
    signal  quad0_rxctrlack3_i              : std_logic;
    signal  quad0_rxdata0_i                 : std_logic_vector(63 downto 0);
    signal  quad0_rxdata1_i                 : std_logic_vector(63 downto 0);
    signal  quad0_rxdata2_i                 : std_logic_vector(63 downto 0);
    signal  quad0_rxdata3_i                 : std_logic_vector(63 downto 0);
    signal  quad0_rxslip0_i                 : std_logic;
    signal  quad0_rxslip1_i                 : std_logic;
    signal  quad0_rxslip2_i                 : std_logic;
    signal  quad0_rxslip3_i                 : std_logic;
    signal  quad0_rxuserclkin0_i            : std_logic;
    signal  quad0_rxuserclkin1_i            : std_logic;
    signal  quad0_rxuserclkin2_i            : std_logic;
    signal  quad0_rxuserclkin3_i            : std_logic;
    signal  quad0_rxuserclkout0_i           : std_logic;
    signal  quad0_rxuserclkout1_i           : std_logic;
    signal  quad0_rxuserclkout2_i           : std_logic;
    signal  quad0_rxuserclkout3_i           : std_logic;
    ------------- Shared Ports - Dynamic Reconfiguration Port (DRP) ------------
    signal  quad0_daddr_i                   : std_logic_vector(15 downto 0);
    signal  quad0_dclk_i                    : std_logic;
    signal  quad0_den_i                     : std_logic;
    signal  quad0_di_i                      : std_logic_vector(15 downto 0);
    signal  quad0_disabledrp_i              : std_logic;
    signal  quad0_drdy_i                    : std_logic;
    signal  quad0_drpdo_i                   : std_logic_vector(15 downto 0);
    signal  quad0_dwe_i                     : std_logic;
    ---------------------------- Shared Ports - Other --------------------------
    signal  quad0_gthinit_i                 : std_logic;
    signal  quad0_gthreset_i                : std_logic;
    signal  quad0_mgmtpcslanesel_i          : std_logic_vector(3 downto 0);
    signal  quad0_mgmtpcsmmdaddr_i          : std_logic_vector(4 downto 0);
    signal  quad0_mgmtpcsrdack_i            : std_logic;
    signal  quad0_mgmtpcsrddata_i           : std_logic_vector(15 downto 0);
    signal  quad0_mgmtpcsregaddr_i          : std_logic_vector(15 downto 0);
    signal  quad0_mgmtpcsregrd_i            : std_logic;
    signal  quad0_mgmtpcsregwr_i            : std_logic;
    signal  quad0_mgmtpcswrdata_i           : std_logic_vector(15 downto 0);
    signal  quad0_refclk_i                  : std_logic;
    ------------------------------- Transmit Ports -----------------------------
    signal  quad0_txbufreset0_i             : std_logic;
    signal  quad0_txbufreset1_i             : std_logic;
    signal  quad0_txbufreset2_i             : std_logic;
    signal  quad0_txbufreset3_i             : std_logic;
    signal  quad0_txctrlack0_i              : std_logic;
    signal  quad0_txctrlack1_i              : std_logic;
    signal  quad0_txctrlack2_i              : std_logic;
    signal  quad0_txctrlack3_i              : std_logic;
    signal  quad0_txdata0_i                 : std_logic_vector(63 downto 0);
    signal  quad0_txdata1_i                 : std_logic_vector(63 downto 0);
    signal  quad0_txdata2_i                 : std_logic_vector(63 downto 0);
    signal  quad0_txdata3_i                 : std_logic_vector(63 downto 0);
    signal  quad0_txuserclkin0_i            : std_logic;
    signal  quad0_txuserclkin1_i            : std_logic;
    signal  quad0_txuserclkin2_i            : std_logic;
    signal  quad0_txuserclkin3_i            : std_logic;
    signal  quad0_txuserclkout0_i           : std_logic;
    signal  quad0_txuserclkout1_i           : std_logic;
    signal  quad0_txuserclkout2_i           : std_logic;
    signal  quad0_txuserclkout3_i           : std_logic;


    --________________________________________________________________________
    --________________________________________________________________________
    --QUAD1   (X1Y1)

    --------------------------------- Powerdown --------------------------------
    signal  quad1_powerdown0_i              : std_logic;
    signal  quad1_powerdown1_i              : std_logic;
    signal  quad1_powerdown2_i              : std_logic;
    signal  quad1_powerdown3_i              : std_logic;
    signal  quad1_rxpowerdown0_i            : std_logic_vector(1 downto 0);
    signal  quad1_rxpowerdown1_i            : std_logic_vector(1 downto 0);
    signal  quad1_rxpowerdown2_i            : std_logic_vector(1 downto 0);
    signal  quad1_rxpowerdown3_i            : std_logic_vector(1 downto 0);
    signal  quad1_txpowerdown0_i            : std_logic_vector(1 downto 0);
    signal  quad1_txpowerdown1_i            : std_logic_vector(1 downto 0);
    signal  quad1_txpowerdown2_i            : std_logic_vector(1 downto 0);
    signal  quad1_txpowerdown3_i            : std_logic_vector(1 downto 0);
    ------------------------------- Receive Ports ------------------------------
    signal  quad1_rxbufreset0_i             : std_logic;
    signal  quad1_rxbufreset1_i             : std_logic;
    signal  quad1_rxbufreset2_i             : std_logic;
    signal  quad1_rxbufreset3_i             : std_logic;
    signal  quad1_rxctrlack0_i              : std_logic;
    signal  quad1_rxctrlack1_i              : std_logic;
    signal  quad1_rxctrlack2_i              : std_logic;
    signal  quad1_rxctrlack3_i              : std_logic;
    signal  quad1_rxdata0_i                 : std_logic_vector(63 downto 0);
    signal  quad1_rxdata1_i                 : std_logic_vector(63 downto 0);
    signal  quad1_rxdata2_i                 : std_logic_vector(63 downto 0);
    signal  quad1_rxdata3_i                 : std_logic_vector(63 downto 0);
    signal  quad1_rxslip0_i                 : std_logic;
    signal  quad1_rxslip1_i                 : std_logic;
    signal  quad1_rxslip2_i                 : std_logic;
    signal  quad1_rxslip3_i                 : std_logic;
    signal  quad1_rxuserclkin0_i            : std_logic;
    signal  quad1_rxuserclkin1_i            : std_logic;
    signal  quad1_rxuserclkin2_i            : std_logic;
    signal  quad1_rxuserclkin3_i            : std_logic;
    signal  quad1_rxuserclkout0_i           : std_logic;
    signal  quad1_rxuserclkout1_i           : std_logic;
    signal  quad1_rxuserclkout2_i           : std_logic;
    signal  quad1_rxuserclkout3_i           : std_logic;
    ------------- Shared Ports - Dynamic Reconfiguration Port (DRP) ------------
    signal  quad1_daddr_i                   : std_logic_vector(15 downto 0);
    signal  quad1_dclk_i                    : std_logic;
    signal  quad1_den_i                     : std_logic;
    signal  quad1_di_i                      : std_logic_vector(15 downto 0);
    signal  quad1_disabledrp_i              : std_logic;
    signal  quad1_drdy_i                    : std_logic;
    signal  quad1_drpdo_i                   : std_logic_vector(15 downto 0);
    signal  quad1_dwe_i                     : std_logic;
    ---------------------------- Shared Ports - Other --------------------------
    signal  quad1_gthinit_i                 : std_logic;
    signal  quad1_gthreset_i                : std_logic;
    signal  quad1_mgmtpcslanesel_i          : std_logic_vector(3 downto 0);
    signal  quad1_mgmtpcsmmdaddr_i          : std_logic_vector(4 downto 0);
    signal  quad1_mgmtpcsrdack_i            : std_logic;
    signal  quad1_mgmtpcsrddata_i           : std_logic_vector(15 downto 0);
    signal  quad1_mgmtpcsregaddr_i          : std_logic_vector(15 downto 0);
    signal  quad1_mgmtpcsregrd_i            : std_logic;
    signal  quad1_mgmtpcsregwr_i            : std_logic;
    signal  quad1_mgmtpcswrdata_i           : std_logic_vector(15 downto 0);
    signal  quad1_refclk_i                  : std_logic;
    ------------------------------- Transmit Ports -----------------------------
    signal  quad1_txbufreset0_i             : std_logic;
    signal  quad1_txbufreset1_i             : std_logic;
    signal  quad1_txbufreset2_i             : std_logic;
    signal  quad1_txbufreset3_i             : std_logic;
    signal  quad1_txctrlack0_i              : std_logic;
    signal  quad1_txctrlack1_i              : std_logic;
    signal  quad1_txctrlack2_i              : std_logic;
    signal  quad1_txctrlack3_i              : std_logic;
    signal  quad1_txdata0_i                 : std_logic_vector(63 downto 0);
    signal  quad1_txdata1_i                 : std_logic_vector(63 downto 0);
    signal  quad1_txdata2_i                 : std_logic_vector(63 downto 0);
    signal  quad1_txdata3_i                 : std_logic_vector(63 downto 0);
    signal  quad1_txuserclkin0_i            : std_logic;
    signal  quad1_txuserclkin1_i            : std_logic;
    signal  quad1_txuserclkin2_i            : std_logic;
    signal  quad1_txuserclkin3_i            : std_logic;
    signal  quad1_txuserclkout0_i           : std_logic;
    signal  quad1_txuserclkout1_i           : std_logic;
    signal  quad1_txuserclkout2_i           : std_logic;
    signal  quad1_txuserclkout3_i           : std_logic;


    --________________________________________________________________________
    --________________________________________________________________________
    --QUAD2   (X1Y2)

    --------------------------------- Powerdown --------------------------------
    signal  quad2_powerdown0_i              : std_logic;
    signal  quad2_powerdown1_i              : std_logic;
    signal  quad2_powerdown2_i              : std_logic;
    signal  quad2_powerdown3_i              : std_logic;
    signal  quad2_rxpowerdown0_i            : std_logic_vector(1 downto 0);
    signal  quad2_rxpowerdown1_i            : std_logic_vector(1 downto 0);
    signal  quad2_rxpowerdown2_i            : std_logic_vector(1 downto 0);
    signal  quad2_rxpowerdown3_i            : std_logic_vector(1 downto 0);
    signal  quad2_txpowerdown0_i            : std_logic_vector(1 downto 0);
    signal  quad2_txpowerdown1_i            : std_logic_vector(1 downto 0);
    signal  quad2_txpowerdown2_i            : std_logic_vector(1 downto 0);
    signal  quad2_txpowerdown3_i            : std_logic_vector(1 downto 0);
    ------------------------------- Receive Ports ------------------------------
    signal  quad2_rxbufreset0_i             : std_logic;
    signal  quad2_rxbufreset1_i             : std_logic;
    signal  quad2_rxbufreset2_i             : std_logic;
    signal  quad2_rxbufreset3_i             : std_logic;
    signal  quad2_rxctrlack0_i              : std_logic;
    signal  quad2_rxctrlack1_i              : std_logic;
    signal  quad2_rxctrlack2_i              : std_logic;
    signal  quad2_rxctrlack3_i              : std_logic;
    signal  quad2_rxdata0_i                 : std_logic_vector(63 downto 0);
    signal  quad2_rxdata1_i                 : std_logic_vector(63 downto 0);
    signal  quad2_rxdata2_i                 : std_logic_vector(63 downto 0);
    signal  quad2_rxdata3_i                 : std_logic_vector(63 downto 0);
    signal  quad2_rxslip0_i                 : std_logic;
    signal  quad2_rxslip1_i                 : std_logic;
    signal  quad2_rxslip2_i                 : std_logic;
    signal  quad2_rxslip3_i                 : std_logic;
    signal  quad2_rxuserclkin0_i            : std_logic;
    signal  quad2_rxuserclkin1_i            : std_logic;
    signal  quad2_rxuserclkin2_i            : std_logic;
    signal  quad2_rxuserclkin3_i            : std_logic;
    signal  quad2_rxuserclkout0_i           : std_logic;
    signal  quad2_rxuserclkout1_i           : std_logic;
    signal  quad2_rxuserclkout2_i           : std_logic;
    signal  quad2_rxuserclkout3_i           : std_logic;
    ------------- Shared Ports - Dynamic Reconfiguration Port (DRP) ------------
    signal  quad2_daddr_i                   : std_logic_vector(15 downto 0);
    signal  quad2_dclk_i                    : std_logic;
    signal  quad2_den_i                     : std_logic;
    signal  quad2_di_i                      : std_logic_vector(15 downto 0);
    signal  quad2_disabledrp_i              : std_logic;
    signal  quad2_drdy_i                    : std_logic;
    signal  quad2_drpdo_i                   : std_logic_vector(15 downto 0);
    signal  quad2_dwe_i                     : std_logic;
    ---------------------------- Shared Ports - Other --------------------------
    signal  quad2_gthinit_i                 : std_logic;
    signal  quad2_gthreset_i                : std_logic;
    signal  quad2_mgmtpcslanesel_i          : std_logic_vector(3 downto 0);
    signal  quad2_mgmtpcsmmdaddr_i          : std_logic_vector(4 downto 0);
    signal  quad2_mgmtpcsrdack_i            : std_logic;
    signal  quad2_mgmtpcsrddata_i           : std_logic_vector(15 downto 0);
    signal  quad2_mgmtpcsregaddr_i          : std_logic_vector(15 downto 0);
    signal  quad2_mgmtpcsregrd_i            : std_logic;
    signal  quad2_mgmtpcsregwr_i            : std_logic;
    signal  quad2_mgmtpcswrdata_i           : std_logic_vector(15 downto 0);
    signal  quad2_refclk_i                  : std_logic;
    ------------------------------- Transmit Ports -----------------------------
    signal  quad2_txbufreset0_i             : std_logic;
    signal  quad2_txbufreset1_i             : std_logic;
    signal  quad2_txbufreset2_i             : std_logic;
    signal  quad2_txbufreset3_i             : std_logic;
    signal  quad2_txctrlack0_i              : std_logic;
    signal  quad2_txctrlack1_i              : std_logic;
    signal  quad2_txctrlack2_i              : std_logic;
    signal  quad2_txctrlack3_i              : std_logic;
    signal  quad2_txdata0_i                 : std_logic_vector(63 downto 0);
    signal  quad2_txdata1_i                 : std_logic_vector(63 downto 0);
    signal  quad2_txdata2_i                 : std_logic_vector(63 downto 0);
    signal  quad2_txdata3_i                 : std_logic_vector(63 downto 0);
    signal  quad2_txuserclkin0_i            : std_logic;
    signal  quad2_txuserclkin1_i            : std_logic;
    signal  quad2_txuserclkin2_i            : std_logic;
    signal  quad2_txuserclkin3_i            : std_logic;
    signal  quad2_txuserclkout0_i           : std_logic;
    signal  quad2_txuserclkout1_i           : std_logic;
    signal  quad2_txuserclkout2_i           : std_logic;
    signal  quad2_txuserclkout3_i           : std_logic;



    signal  quad0_tx_pcs_resetdone0_i       : std_logic;
    signal  quad0_rx_pcs_cdr_resetdone0_i   : std_logic;
    signal  quad0_tx_pcs_reset0_i           : std_logic;
    signal  quad0_rx_pcs_cdr_reset0_i       : std_logic;
    signal  quad0_tx_system_reset0_c        : std_logic;
    signal  quad0_rx_system_reset0_c        : std_logic;
    signal  quad0_tx_pcs_resetdone1_i       : std_logic;
    signal  quad0_rx_pcs_cdr_resetdone1_i   : std_logic;
    signal  quad0_tx_pcs_reset1_i           : std_logic;
    signal  quad0_rx_pcs_cdr_reset1_i       : std_logic;
    signal  quad0_tx_system_reset1_c        : std_logic;
    signal  quad0_rx_system_reset1_c        : std_logic;
    signal  quad0_tx_pcs_resetdone2_i       : std_logic;
    signal  quad0_rx_pcs_cdr_resetdone2_i   : std_logic;
    signal  quad0_tx_pcs_reset2_i           : std_logic;
    signal  quad0_rx_pcs_cdr_reset2_i       : std_logic;
    signal  quad0_tx_system_reset2_c        : std_logic;
    signal  quad0_rx_system_reset2_c        : std_logic;
    signal  quad0_tx_pcs_resetdone3_i       : std_logic;
    signal  quad0_rx_pcs_cdr_resetdone3_i   : std_logic;
    signal  quad0_tx_pcs_reset3_i           : std_logic;
    signal  quad0_rx_pcs_cdr_reset3_i       : std_logic;
    signal  quad0_tx_system_reset3_c        : std_logic;
    signal  quad0_rx_system_reset3_c        : std_logic;
    signal  quad1_tx_pcs_resetdone0_i       : std_logic;
    signal  quad1_rx_pcs_cdr_resetdone0_i   : std_logic;
    signal  quad1_tx_pcs_reset0_i           : std_logic;
    signal  quad1_rx_pcs_cdr_reset0_i       : std_logic;
    signal  quad1_tx_system_reset0_c        : std_logic;
    signal  quad1_rx_system_reset0_c        : std_logic;
    signal  quad1_tx_pcs_resetdone1_i       : std_logic;
    signal  quad1_rx_pcs_cdr_resetdone1_i   : std_logic;
    signal  quad1_tx_pcs_reset1_i           : std_logic;
    signal  quad1_rx_pcs_cdr_reset1_i       : std_logic;
    signal  quad1_tx_system_reset1_c        : std_logic;
    signal  quad1_rx_system_reset1_c        : std_logic;
    signal  quad1_tx_pcs_resetdone2_i       : std_logic;
    signal  quad1_rx_pcs_cdr_resetdone2_i   : std_logic;
    signal  quad1_tx_pcs_reset2_i           : std_logic;
    signal  quad1_rx_pcs_cdr_reset2_i       : std_logic;
    signal  quad1_tx_system_reset2_c        : std_logic;
    signal  quad1_rx_system_reset2_c        : std_logic;
    signal  quad1_tx_pcs_resetdone3_i       : std_logic;
    signal  quad1_rx_pcs_cdr_resetdone3_i   : std_logic;
    signal  quad1_tx_pcs_reset3_i           : std_logic;
    signal  quad1_rx_pcs_cdr_reset3_i       : std_logic;
    signal  quad1_tx_system_reset3_c        : std_logic;
    signal  quad1_rx_system_reset3_c        : std_logic;
    signal  quad2_tx_pcs_resetdone0_i       : std_logic;
    signal  quad2_rx_pcs_cdr_resetdone0_i   : std_logic;
    signal  quad2_tx_pcs_reset0_i           : std_logic;
    signal  quad2_rx_pcs_cdr_reset0_i       : std_logic;
    signal  quad2_tx_system_reset0_c        : std_logic;
    signal  quad2_rx_system_reset0_c        : std_logic;
    signal  quad2_tx_pcs_resetdone1_i       : std_logic;
    signal  quad2_rx_pcs_cdr_resetdone1_i   : std_logic;
    signal  quad2_tx_pcs_reset1_i           : std_logic;
    signal  quad2_rx_pcs_cdr_reset1_i       : std_logic;
    signal  quad2_tx_system_reset1_c        : std_logic;
    signal  quad2_rx_system_reset1_c        : std_logic;
    signal  quad2_tx_pcs_resetdone2_i       : std_logic;
    signal  quad2_rx_pcs_cdr_resetdone2_i   : std_logic;
    signal  quad2_tx_pcs_reset2_i           : std_logic;
    signal  quad2_rx_pcs_cdr_reset2_i       : std_logic;
    signal  quad2_tx_system_reset2_c        : std_logic;
    signal  quad2_rx_system_reset2_c        : std_logic;
    signal  quad2_tx_pcs_resetdone3_i       : std_logic;
    signal  quad2_rx_pcs_cdr_resetdone3_i   : std_logic;
    signal  quad2_tx_pcs_reset3_i           : std_logic;
    signal  quad2_rx_pcs_cdr_reset3_i       : std_logic;
    signal  quad2_tx_system_reset3_c        : std_logic;
    signal  quad2_rx_system_reset3_c        : std_logic;
    signal  tied_to_ground_i                : std_logic;
    signal  tied_to_ground_vec_i            : std_logic_vector(127 downto 0);
    signal  tied_to_vcc_i                   : std_logic;
    signal  tied_to_vcc_vec_i               : std_logic_vector(7 downto 0);
    signal  drp_clk_in_i                    : std_logic; 
    
SIGNAL quad0_gthinitdone_i	: STD_LOGIC;
SIGNAL quad1_gthinitdone_i	: STD_LOGIC;
SIGNAL quad2_gthinitdone_i	: STD_LOGIC;
    
-- Synthetised DRP clock (39MHz)
--SIGNAL drp_cntr			: STD_LOGIC;
--SIGNAL DRP_CE				: STD_LOGIC;
--SIGNAL DRP_CLK				: STD_LOGIC;

-- Synthetised 312.5MHz MAC clock
  SIGNAL quad0_refclk_o	: STD_LOGIC;
  signal mmcRX_clkin1      : std_logic;
  SIGNAL mmcRX_clk_out1	: STD_LOGIC;
  SIGNAL mmcRX_locked		: STD_LOGIC;
  SIGNAL mmc_RESET		: STD_LOGIC;
  -- Output clock buffering / unused connectors
  signal mmcRX_clkfbout         : std_logic;
  signal mmcRX_clkfbout_buf     : std_logic;
  signal mmcRX_clkfboutb_unused : std_logic;
  signal mmcRX_clkout0          : std_logic;
  signal mmcRX_clkout0b_unused  : std_logic;
  signal mmcRX_clkout1_unused   : std_logic;
  signal mmcRX_clkout1b_unused  : std_logic;
  signal mmcRX_clkout2_unused   : std_logic;
  signal mmcRX_clkout2b_unused  : std_logic;
  signal mmcRX_clkout3_unused   : std_logic;
  signal mmcRX_clkout3b_unused  : std_logic;
  signal mmcRX_clkout4_unused   : std_logic;
  signal mmcRX_clkout5_unused   : std_logic;
  signal mmcRX_clkout6_unused   : std_logic;
  -- Dynamic programming unused signals
  signal mmcRX_do_unused        : std_logic_vector(15 downto 0);
  signal mmcRX_drdy_unused      : std_logic;
  -- Dynamic phase shift unused signals
  signal mmcRX_psdone_unused    : std_logic;
  -- Unused status signals
  signal mmcRX_clkfbstopped_unused : std_logic;
  signal mmcRX_clkinstopped_unused : std_logic; 


  signal mmcTX_clkin1      : std_logic;
  SIGNAL mmcTX_clk_out1	: STD_LOGIC;
  SIGNAL mmcTX_locked		: STD_LOGIC;
  -- Output clock buffering / unused connectors
  signal mmcTX_clkfbout         : std_logic;
  signal mmcTX_clkfbout_buf     : std_logic;
  signal mmcTX_clkfboutb_unused : std_logic;
  signal mmcTX_clkout0          : std_logic;
  signal mmcTX_clkout0b_unused  : std_logic;
  signal mmcTX_clkout1_unused   : std_logic;
  signal mmcTX_clkout1b_unused  : std_logic;
  signal mmcTX_clkout2_unused   : std_logic;
  signal mmcTX_clkout2b_unused  : std_logic;
  signal mmcTX_clkout3_unused   : std_logic;
  signal mmcTX_clkout3b_unused  : std_logic;
  signal mmcTX_clkout4_unused   : std_logic;
  signal mmcTX_clkout5_unused   : std_logic;
  signal mmcTX_clkout6_unused   : std_logic;
  -- Dynamic programming unused signals
  signal mmcTX_do_unused        : std_logic_vector(15 downto 0);
  signal mmcTX_drdy_unused      : std_logic;
  -- Dynamic phase shift unused signals
  signal mmcTX_psdone_unused    : std_logic;
  -- Unused status signals
  signal mmcTX_clkfbstopped_unused : std_logic;
  signal mmcTX_clkinstopped_unused : std_logic; 

    
--**************************** Main Body of Code *******************************

begin

    --  Static signal Assigments
    tied_to_ground_i                             <= '0';
    tied_to_ground_vec_i                         <= x"00000000000000000000000000000000";
    tied_to_vcc_i                                <= '1';
    tied_to_vcc_vec_i                            <= x"ff";


    ------------------------------ DRP Clock ----------------------------------
    
	drp_clk_in_i	<= DRP_CLK;
  
    -----------------------Dedicated GTH Reference Clock Inputs ---------------
    -- The dedicated reference clock inputs you selected in the GUI are implemented using
    -- IBUFDS_GTHE1 instances.
    --
    -- In the UCF file for this example design, you will see that each of
    -- these IBUFDS_GTHE1 instances has been LOCed to a particular set of pins. By LOCing to these
    -- locations, we tell the tools to use the dedicated input buffers to the GTH reference
    -- clock network, rather than general purpose IOs. To select other pins, consult the 
    -- Implementation chapter of UG371, or rerun the wizard.
    --
    -- This network is the only  option for providing clocks to the GTH transceivers.
    
    quad0_refclk_ibufds_i : IBUFDS_GTHE1
    port map
    (
        O                               =>      quad0_refclk_i,
        I                               =>      Q0_REFCLK_P,
        IB                              =>      Q0_REFCLK_N
    );
    
  
    quad1_refclk_ibufds_i : IBUFDS_GTHE1
    port map
    (
        O                               =>      quad1_refclk_i,
        I                               =>      Q1_REFCLK_P,
        IB                              =>      Q1_REFCLK_N
    );
    
  
    quad2_refclk_ibufds_i : IBUFDS_GTHE1
    port map
    (
        O                               =>      quad2_refclk_i,
        I                               =>      Q2_REFCLK_P,
        IB                              =>      Q2_REFCLK_N
    );


    ----------------------------------- User Clocks ---------------------------
    
    --   Use caution when sharing RXUSERCLKOUT as a clock source - these clocks can typically only be shared if all
    --   the channels using the clock are receiving data from TX channels that share a reference clock 
    --   source with each other.   


-- PCS clocks (161.133MHz)
	clk_GTH_rx	<= quad0_rxuserclkin0_i;
	clk_GTH_tx	<= quad0_txuserclkin0_i;

-- common MAC clock (312.5MHz)
	clk_MAC_RX		<= mmcRX_CLK_OUT1;
	clk_MAC_TX		<= mmcTX_CLK_OUT1;
--
	mmcRX_clkin1		<= quad0_rxuserclkin0_i; -- quad0_refclk_o;

-- WARNING! MMCM freq synthesis simulation with fractional DIVIDE is broken in ISE <= 14.4 (hopefully implementation is right)

  mmcRX_clk312 : MMCM_ADV
  generic map
   (BANDWIDTH            => "OPTIMIZED",
    CLKOUT4_CASCADE      => FALSE,
    CLOCK_HOLD           => FALSE,
    COMPENSATION         => "ZHOLD",
    STARTUP_WAIT         => FALSE,
-- Uncomment for faster MAC clock!
--    DIVCLK_DIVIDE        => 2,
--    CLKFBOUT_MULT_F      => 17.000,
--    CLKOUT0_DIVIDE_F     => 4.375,
--
    DIVCLK_DIVIDE        => 11,
    CLKFBOUT_MULT_F      => 64.000,
    CLKOUT0_DIVIDE_F     => 3.000,
--
    CLKFBOUT_PHASE       => 0.000,
    CLKFBOUT_USE_FINE_PS => FALSE,
    CLKOUT0_PHASE        => 0.000,
    CLKOUT0_DUTY_CYCLE   => 0.500,
    CLKOUT0_USE_FINE_PS  => FALSE,
    CLKIN1_PERIOD        => 6.206,
    REF_JITTER1          => 0.010)
  port map
    -- Output clocks
   (CLKFBOUT            => mmcRX_clkfbout,
    CLKFBOUTB           => mmcRX_clkfboutb_unused,
    CLKOUT0             => mmcRX_clkout0,
    CLKOUT0B            => mmcRX_clkout0b_unused,
    CLKOUT1             => mmcRX_clkout1_unused,
    CLKOUT1B            => mmcRX_clkout1b_unused,
    CLKOUT2             => mmcRX_clkout2_unused,
    CLKOUT2B            => mmcRX_clkout2b_unused,
    CLKOUT3             => mmcRX_clkout3_unused,
    CLKOUT3B            => mmcRX_clkout3b_unused,
    CLKOUT4             => mmcRX_clkout4_unused,
    CLKOUT5             => mmcRX_clkout5_unused,
    CLKOUT6             => mmcRX_clkout6_unused,
    -- Input clock control
    CLKFBIN             => mmcRX_clkfbout_buf,
    CLKIN1              => mmcRX_clkin1,
    CLKIN2              => '0',
    -- Tied to always select the primary input clock
    CLKINSEL            => '1',
    -- Ports for dynamic reconfiguration
    DADDR               => (others => '0'),
    DCLK                => '0',
    DEN                 => '0',
    DI                  => (others => '0'),
    DO                  => mmcRX_do_unused,
    DRDY                => mmcRX_drdy_unused,
    DWE                 => '0',
    -- Ports for dynamic phase shift
    PSCLK               => '0',
    PSEN                => '0',
    PSINCDEC            => '0',
    PSDONE              => mmcRX_psdone_unused,
    -- Other control and status signals
    LOCKED              => mmcRX_LOCKED,
    CLKINSTOPPED        => mmcRX_clkinstopped_unused,
    CLKFBSTOPPED        => mmcRX_clkfbstopped_unused,
    PWRDWN              => '0',
    RST                 => mmc_RESET
	); 

--
mmc_RESET		<= NOT quad0_gthinitdone_i;

  -- Output buffering
--  clkfrx_buf : BUFH
--  port map(
--	I => mmcRX_clkfbout,
--	O => mmcRX_clkfbout_buf
--	);

-- Try the low jitter internal feedback
mmcRX_clkfbout_buf	<= mmcRX_clkfbout;

  clkout1RX_buf : BUFG
  port map(
	I   => mmcRX_clkout0,
	O   => mmcRX_CLK_OUT1
	);

-- TX MAC clock

	mmcTX_clkin1		<= quad0_txuserclkin0_i; -- quad0_refclk_o;

  mmcTX_clk312 : MMCM_ADV
  generic map
   (BANDWIDTH            => "OPTIMIZED",
    CLKOUT4_CASCADE      => FALSE,
    CLOCK_HOLD           => FALSE,
    COMPENSATION         => "ZHOLD",
    STARTUP_WAIT         => FALSE,
-- Uncomment for faster MAC clock!
--    DIVCLK_DIVIDE        => 2,
--    CLKFBOUT_MULT_F      => 17.000,
--    CLKOUT0_DIVIDE_F     => 4.375,
--
    DIVCLK_DIVIDE        => 11,
    CLKFBOUT_MULT_F      => 64.000,
    CLKOUT0_DIVIDE_F     => 3.000,
--
    CLKFBOUT_PHASE       => 0.000,
    CLKFBOUT_USE_FINE_PS => FALSE,
    CLKOUT0_PHASE        => 0.000,
    CLKOUT0_DUTY_CYCLE   => 0.500,
    CLKOUT0_USE_FINE_PS  => FALSE,
    CLKIN1_PERIOD        => 6.206,
    REF_JITTER1          => 0.010)
  port map
    -- Output clocks
   (CLKFBOUT            => mmcTX_clkfbout,
    CLKFBOUTB           => mmcTX_clkfboutb_unused,
    CLKOUT0             => mmcTX_clkout0,
    CLKOUT0B            => mmcTX_clkout0b_unused,
    CLKOUT1             => mmcTX_clkout1_unused,
    CLKOUT1B            => mmcTX_clkout1b_unused,
    CLKOUT2             => mmcTX_clkout2_unused,
    CLKOUT2B            => mmcTX_clkout2b_unused,
    CLKOUT3             => mmcTX_clkout3_unused,
    CLKOUT3B            => mmcTX_clkout3b_unused,
    CLKOUT4             => mmcTX_clkout4_unused,
    CLKOUT5             => mmcTX_clkout5_unused,
    CLKOUT6             => mmcTX_clkout6_unused,
    -- Input clock control
    CLKFBIN             => mmcTX_clkfbout_buf,
    CLKIN1              => mmcTX_clkin1,
    CLKIN2              => '0',
    -- Tied to always select the primary input clock
    CLKINSEL            => '1',
    -- Ports for dynamic reconfiguration
    DADDR               => (others => '0'),
    DCLK                => '0',
    DEN                 => '0',
    DI                  => (others => '0'),
    DO                  => mmcTX_do_unused,
    DRDY                => mmcTX_drdy_unused,
    DWE                 => '0',
    -- Ports for dynamic phase shift
    PSCLK               => '0',
    PSEN                => '0',
    PSINCDEC            => '0',
    PSDONE              => mmcTX_psdone_unused,
    -- Other control and status signals
    LOCKED              => mmcTX_LOCKED,
    CLKINSTOPPED        => mmcTX_clkinstopped_unused,
    CLKFBSTOPPED        => mmcTX_clkfbstopped_unused,
    PWRDWN              => '0',
    RST                 => mmc_RESET
	); 

 -- Output buffering
--  clkftx_buf : BUFH
--  port map(
--	I => mmcTX_clkfbout,
--	O => mmcTX_clkfbout_buf
--	);

mmcTX_clkfbout_buf	<= mmcTX_clkfbout;

  clkout1TX_buf : BUFG
  port map(
	I   => mmcTX_clkout0,
	O   => mmcTX_CLK_OUT1
	);


-- DATA strappings:
-- OP2 = N.C. (quad2_data2_i)
-- OP3 = REFCLK(tx), RXMCLK(rx) (quad2_data3_i)


	DOUT_GTH		<= quad2_rxdata1_i &
				quad2_rxdata0_i &
				quad1_rxdata3_i &
				quad1_rxdata2_i &
				quad1_rxdata1_i &
				quad1_rxdata0_i &
				quad0_rxdata3_i &
				quad0_rxdata2_i &
				quad0_rxdata1_i &
				quad0_rxdata0_i;

	quad0_txdata0_i	<= DIN_GTH(1*64-1 downto 0*64);
	quad0_txdata1_i	<= DIN_GTH(2*64-1 downto 1*64);
	quad0_txdata2_i	<= DIN_GTH(3*64-1 downto 2*64);
	quad0_txdata3_i	<= DIN_GTH(4*64-1 downto 3*64);
	--
	quad1_txdata0_i	<= DIN_GTH(5*64-1 downto 4*64);
	quad1_txdata1_i	<= DIN_GTH(6*64-1 downto 5*64);
	quad1_txdata2_i	<= DIN_GTH(7*64-1 downto 6*64);
	quad1_txdata3_i	<= DIN_GTH(8*64-1 downto 7*64);
	--
	quad2_txdata0_i	<= DIN_GTH(9*64-1 downto 8*64);
	quad2_txdata1_i	<= DIN_GTH(10*64-1 downto 9*64);
	
	-- REFCLK! [CFP MSA HW spec 1.40; 4.4.4 ] (needed 4 transmission)
	quad2_txdata3_i	<= X"00000000FFFFFFFF"; -- = 1/64 of host lane-rate
	
-- Slippery when wet
	quad0_rxslip0_i	<= '0'; --slip_GTH(0);
	quad0_rxslip1_i	<= '0'; --slip_GTH(1);
	quad0_rxslip2_i	<= '0'; --slip_GTH(2);
	quad0_rxslip3_i	<= '0'; --slip_GTH(3);
	--
	quad1_rxslip0_i	<= '0'; --slip_GTH(4);
	quad1_rxslip1_i	<= '0'; --slip_GTH(5);
	quad1_rxslip2_i	<= '0'; --slip_GTH(6);
	quad1_rxslip3_i	<= '0'; --slip_GTH(7);
	--
	quad2_rxslip0_i	<= '0'; --slip_GTH(8);
	quad2_rxslip1_i	<= '0'; --slip_GTH(9);


    quad0_txuserclkout0_bufg0_i : BUFG
    port map
    (
        I                               =>      quad0_txuserclkout0_i,
        O                               =>      quad0_txuserclkin0_i
    );


    quad0_rxuserclkout0_bufg1_i : BUFG
    port map
    (
        I                               =>      quad0_rxuserclkout0_i,
        O                               =>      quad0_rxuserclkin0_i
    );



    -- assign resets for frame_gen/check modules
--    quad0_rx_system_reset0_c                     <= not quad0_rx_pcs_cdr_resetdone0_i;
--    quad0_tx_system_reset0_c                     <= not quad0_tx_pcs_resetdone0_i;
--    quad0_rx_system_reset1_c                     <= not quad0_rx_pcs_cdr_resetdone1_i;
--    quad0_tx_system_reset1_c                     <= not quad0_tx_pcs_resetdone1_i;
--    quad0_rx_system_reset2_c                     <= not quad0_rx_pcs_cdr_resetdone2_i;
--    quad0_tx_system_reset2_c                     <= not quad0_tx_pcs_resetdone2_i;
--    quad0_rx_system_reset3_c                     <= not quad0_rx_pcs_cdr_resetdone3_i;
--    quad0_tx_system_reset3_c                     <= not quad0_tx_pcs_resetdone3_i;
--    quad1_rx_system_reset0_c                     <= not quad1_rx_pcs_cdr_resetdone0_i;
--    quad1_tx_system_reset0_c                     <= not quad1_tx_pcs_resetdone0_i;
--    quad1_rx_system_reset1_c                     <= not quad1_rx_pcs_cdr_resetdone1_i;
--    quad1_tx_system_reset1_c                     <= not quad1_tx_pcs_resetdone1_i;
--    quad1_rx_system_reset2_c                     <= not quad1_rx_pcs_cdr_resetdone2_i;
--    quad1_tx_system_reset2_c                     <= not quad1_tx_pcs_resetdone2_i;
--    quad1_rx_system_reset3_c                     <= not quad1_rx_pcs_cdr_resetdone3_i;
--    quad1_tx_system_reset3_c                     <= not quad1_tx_pcs_resetdone3_i;
--    quad2_rx_system_reset0_c                     <= not quad2_rx_pcs_cdr_resetdone0_i;
--    quad2_tx_system_reset0_c                     <= not quad2_tx_pcs_resetdone0_i;
--    quad2_rx_system_reset1_c                     <= not quad2_rx_pcs_cdr_resetdone1_i;
--    quad2_tx_system_reset1_c                     <= not quad2_tx_pcs_resetdone1_i;
--    quad2_rx_system_reset2_c                     <= not quad2_rx_pcs_cdr_resetdone2_i;
--    quad2_tx_system_reset2_c                     <= not quad2_tx_pcs_resetdone2_i;
--    quad2_rx_system_reset3_c                     <= not quad2_rx_pcs_cdr_resetdone3_i;
--    quad2_tx_system_reset3_c                     <= not quad2_tx_pcs_resetdone3_i;

    -- sync inputs to GTH
    quad0_daddr_i                       <=  tied_to_ground_vec_i(15 downto 0);
    quad0_den_i                         <=  tied_to_ground_i;
    quad0_di_i                          <=  tied_to_ground_vec_i(15 downto 0);
    quad0_dwe_i                         <=  tied_to_ground_i;

    -- sync inputs to GTH
    quad1_daddr_i                       <=  tied_to_ground_vec_i(15 downto 0);
    quad1_den_i                         <=  tied_to_ground_i;
    quad1_di_i                          <=  tied_to_ground_vec_i(15 downto 0);
    quad1_dwe_i                         <=  tied_to_ground_i;

    -- sync inputs to GTH
    quad2_daddr_i                       <=  tied_to_ground_vec_i(15 downto 0);
    quad2_den_i                         <=  tied_to_ground_i;
    quad2_di_i                          <=  tied_to_ground_vec_i(15 downto 0);
    quad2_dwe_i                         <=  tied_to_ground_i;

    quad0_gthreset_i                             <= tied_to_ground_i;
    quad0_gthinit_i                              <= tied_to_ground_i;
    quad0_tx_pcs_reset0_i                        <= tied_to_ground_i;
    quad0_rx_pcs_cdr_reset0_i                    <= CDR_RST; --tied_to_ground_i;
    quad0_tx_pcs_reset1_i                        <= tied_to_ground_i;
    quad0_rx_pcs_cdr_reset1_i                    <= CDR_RST; --tied_to_ground_i;
    quad0_tx_pcs_reset2_i                        <= tied_to_ground_i;
    quad0_rx_pcs_cdr_reset2_i                    <= CDR_RST; --tied_to_ground_i;
    quad0_tx_pcs_reset3_i                        <= tied_to_ground_i;
    quad0_rx_pcs_cdr_reset3_i                    <= CDR_RST; --tied_to_ground_i;
    quad0_powerdown0_i                           <= tied_to_ground_i;
    quad0_txpowerdown0_i                         <= tied_to_ground_vec_i(1 downto 0);
    quad0_txbufreset0_i                          <= tied_to_ground_i;
    quad0_rxpowerdown0_i                         <= tied_to_ground_vec_i(1 downto 0);
    quad0_rxbufreset0_i                          <= tied_to_ground_i;
    quad0_powerdown1_i                           <= tied_to_ground_i;
    quad0_txpowerdown1_i                         <= tied_to_ground_vec_i(1 downto 0);
    quad0_txbufreset1_i                          <= tied_to_ground_i;
    quad0_rxpowerdown1_i                         <= tied_to_ground_vec_i(1 downto 0);
    quad0_rxbufreset1_i                          <= tied_to_ground_i;
    quad0_powerdown2_i                           <= tied_to_ground_i;
    quad0_txpowerdown2_i                         <= tied_to_ground_vec_i(1 downto 0);
    quad0_txbufreset2_i                          <= tied_to_ground_i;
    quad0_rxpowerdown2_i                         <= tied_to_ground_vec_i(1 downto 0);
    quad0_rxbufreset2_i                          <= tied_to_ground_i;
    quad0_powerdown3_i                           <= tied_to_ground_i;
    quad0_txpowerdown3_i                         <= tied_to_ground_vec_i(1 downto 0);
    quad0_txbufreset3_i                          <= tied_to_ground_i;
    quad0_rxpowerdown3_i                         <= tied_to_ground_vec_i(1 downto 0);
    quad0_rxbufreset3_i                          <= tied_to_ground_i;
    quad1_gthreset_i                             <= tied_to_ground_i;
    quad1_gthinit_i                              <= tied_to_ground_i;
    quad1_tx_pcs_reset0_i                        <= tied_to_ground_i;
    quad1_rx_pcs_cdr_reset0_i                    <= CDR_RST; --tied_to_ground_i;
    quad1_tx_pcs_reset1_i                        <= tied_to_ground_i;
    quad1_rx_pcs_cdr_reset1_i                    <= CDR_RST; --tied_to_ground_i;
    quad1_tx_pcs_reset2_i                        <= tied_to_ground_i;
    quad1_rx_pcs_cdr_reset2_i                    <= CDR_RST; --tied_to_ground_i;
    quad1_tx_pcs_reset3_i                        <= tied_to_ground_i;
    quad1_rx_pcs_cdr_reset3_i                    <= CDR_RST; --tied_to_ground_i;
    quad1_powerdown0_i                           <= tied_to_ground_i;
    quad1_txpowerdown0_i                         <= tied_to_ground_vec_i(1 downto 0);
    quad1_txbufreset0_i                          <= tied_to_ground_i;
    quad1_rxpowerdown0_i                         <= tied_to_ground_vec_i(1 downto 0);
    quad1_rxbufreset0_i                          <= tied_to_ground_i;
    quad1_powerdown1_i                           <= tied_to_ground_i;
    quad1_txpowerdown1_i                         <= tied_to_ground_vec_i(1 downto 0);
    quad1_txbufreset1_i                          <= tied_to_ground_i;
    quad1_rxpowerdown1_i                         <= tied_to_ground_vec_i(1 downto 0);
    quad1_rxbufreset1_i                          <= tied_to_ground_i;
    quad1_powerdown2_i                           <= tied_to_ground_i;
    quad1_txpowerdown2_i                         <= tied_to_ground_vec_i(1 downto 0);
    quad1_txbufreset2_i                          <= tied_to_ground_i;
    quad1_rxpowerdown2_i                         <= tied_to_ground_vec_i(1 downto 0);
    quad1_rxbufreset2_i                          <= tied_to_ground_i;
    quad1_powerdown3_i                           <= tied_to_ground_i;
    quad1_txpowerdown3_i                         <= tied_to_ground_vec_i(1 downto 0);
    quad1_txbufreset3_i                          <= tied_to_ground_i;
    quad1_rxpowerdown3_i                         <= tied_to_ground_vec_i(1 downto 0);
    quad1_rxbufreset3_i                          <= tied_to_ground_i;
    quad2_gthreset_i                             <= tied_to_ground_i;
    quad2_gthinit_i                              <= tied_to_ground_i;
    quad2_tx_pcs_reset0_i                        <= tied_to_ground_i;
    quad2_rx_pcs_cdr_reset0_i                    <= CDR_RST; --tied_to_ground_i;
    quad2_tx_pcs_reset1_i                        <= tied_to_ground_i;
    quad2_rx_pcs_cdr_reset1_i                    <= CDR_RST; --tied_to_ground_i;
    quad2_tx_pcs_reset2_i                        <= tied_to_ground_i;
    quad2_rx_pcs_cdr_reset2_i                    <= CDR_RST; --tied_to_ground_i;
    quad2_tx_pcs_reset3_i                        <= tied_to_ground_i;
    quad2_rx_pcs_cdr_reset3_i                    <= CDR_RST; --tied_to_ground_i;
    quad2_powerdown0_i                           <= tied_to_ground_i;
    quad2_txpowerdown0_i                         <= tied_to_ground_vec_i(1 downto 0);
    quad2_txbufreset0_i                          <= tied_to_ground_i;
    quad2_rxpowerdown0_i                         <= tied_to_ground_vec_i(1 downto 0);
    quad2_rxbufreset0_i                          <= tied_to_ground_i;
    quad2_powerdown1_i                           <= tied_to_ground_i;
    quad2_txpowerdown1_i                         <= tied_to_ground_vec_i(1 downto 0);
    quad2_txbufreset1_i                          <= tied_to_ground_i;
    quad2_rxpowerdown1_i                         <= tied_to_ground_vec_i(1 downto 0);
    quad2_rxbufreset1_i                          <= tied_to_ground_i;
    quad2_powerdown2_i                           <= tied_to_ground_i;
    quad2_txpowerdown2_i                         <= tied_to_ground_vec_i(1 downto 0);
    quad2_txbufreset2_i                          <= tied_to_ground_i;
    quad2_rxpowerdown2_i                         <= tied_to_ground_vec_i(1 downto 0);
    quad2_rxbufreset2_i                          <= tied_to_ground_i;
    quad2_powerdown3_i                           <= tied_to_ground_i;
    quad2_txpowerdown3_i                         <= tied_to_ground_vec_i(1 downto 0);
    quad2_txbufreset3_i                          <= tied_to_ground_i;
    quad2_rxpowerdown3_i                         <= tied_to_ground_vec_i(1 downto 0);
    quad2_rxbufreset3_i                          <= tied_to_ground_i; 


    ----------------------------- The GTH Wrapper -----------------------------
    
    -- Use the instantiation template in the example directory to add the GTH wrapper to your design.
    -- In this example, the wrapper is wired up for basic operation with a frame generator and frame 
    -- checker. The GTHs will reset, then attempt to align and transmit data. If channel bonding is 
    -- enabled, bonding should occur after alignment.
    -- THE PCS_RESET and RESETDONE signals no not appear on the GTHE1_QUAD primative. They are povided
    -- by the wrapper for ease of use. 
    -- TX_PCS_RESET and TX_PCS_RESETDONE are synchronous to TXUSERCLK while RX_PCS_RESET 
    -- and RX_PCS_CDR_RESETDONE are synchronous to RXUSERCLK.

GTH_INITDONE	<= quad0_gthinitdone_i AND quad1_gthinitdone_i AND quad2_gthinitdone_i AND mmcRX_LOCKED AND mmcTX_LOCKED;
    
    v6_gthwizard_v1_11_caui_i : v6_gthwizard_v1_11_caui
    generic map
    (
        WRAPPER_SIM_GTHRESET_SPEEDUP    =>      1 --EXAMPLE_SIM_GTHRESET_SPEEDUP
    )
    port map
    (

        --_____________________________________________________________________
        --_____________________________________________________________________
        --QUAD0  (X1Y0)
        QUAD0_TX_PCS_RESETDONE0_OUT     =>      quad0_tx_pcs_resetdone0_i,
        QUAD0_RX_PCS_CDR_RESETDONE0_OUT =>      quad0_rx_pcs_cdr_resetdone0_i,
        QUAD0_TX_PCS_RESET0_IN          =>      quad0_tx_pcs_reset0_i,
        QUAD0_RX_PCS_CDR_RESET0_IN      =>      quad0_rx_pcs_cdr_reset0_i,
        QUAD0_TX_PCS_RESETDONE1_OUT     =>      quad0_tx_pcs_resetdone1_i,
        QUAD0_RX_PCS_CDR_RESETDONE1_OUT =>      quad0_rx_pcs_cdr_resetdone1_i,
        QUAD0_TX_PCS_RESET1_IN          =>      quad0_tx_pcs_reset1_i,
        QUAD0_RX_PCS_CDR_RESET1_IN      =>      quad0_rx_pcs_cdr_reset1_i,
        QUAD0_TX_PCS_RESETDONE2_OUT     =>      quad0_tx_pcs_resetdone2_i,
        QUAD0_RX_PCS_CDR_RESETDONE2_OUT =>      quad0_rx_pcs_cdr_resetdone2_i,
        QUAD0_TX_PCS_RESET2_IN          =>      quad0_tx_pcs_reset2_i,
        QUAD0_RX_PCS_CDR_RESET2_IN      =>      quad0_rx_pcs_cdr_reset2_i,
        QUAD0_TX_PCS_RESETDONE3_OUT     =>      quad0_tx_pcs_resetdone3_i,
        QUAD0_RX_PCS_CDR_RESETDONE3_OUT =>      quad0_rx_pcs_cdr_resetdone3_i,
        QUAD0_TX_PCS_RESET3_IN          =>      quad0_tx_pcs_reset3_i,
        QUAD0_RX_PCS_CDR_RESET3_IN      =>      quad0_rx_pcs_cdr_reset3_i,
        QUAD0_GTHINITDONE_OUT           =>      quad0_gthinitdone_i, --open,
        --------------------------------- Powerdown --------------------------------
        QUAD0_POWERDOWN0_IN             =>      quad0_powerdown0_i,
        QUAD0_POWERDOWN1_IN             =>      quad0_powerdown1_i,
        QUAD0_POWERDOWN2_IN             =>      quad0_powerdown2_i,
        QUAD0_POWERDOWN3_IN             =>      quad0_powerdown3_i,
        QUAD0_RXPOWERDOWN0_IN           =>      quad0_rxpowerdown0_i,
        QUAD0_RXPOWERDOWN1_IN           =>      quad0_rxpowerdown1_i,
        QUAD0_RXPOWERDOWN2_IN           =>      quad0_rxpowerdown2_i,
        QUAD0_RXPOWERDOWN3_IN           =>      quad0_rxpowerdown3_i,
        QUAD0_TXPOWERDOWN0_IN           =>      quad0_txpowerdown0_i,
        QUAD0_TXPOWERDOWN1_IN           =>      quad0_txpowerdown1_i,
        QUAD0_TXPOWERDOWN2_IN           =>      quad0_txpowerdown2_i,
        QUAD0_TXPOWERDOWN3_IN           =>      quad0_txpowerdown3_i,
        ------------------------------- Receive Ports ------------------------------
        QUAD0_RXBUFRESET0_IN            =>      quad0_rxbufreset0_i,
        QUAD0_RXBUFRESET1_IN            =>      quad0_rxbufreset1_i,
        QUAD0_RXBUFRESET2_IN            =>      quad0_rxbufreset2_i,
        QUAD0_RXBUFRESET3_IN            =>      quad0_rxbufreset3_i,
        QUAD0_RXCTRLACK0_OUT            =>      quad0_rxctrlack0_i,
        QUAD0_RXCTRLACK1_OUT            =>      quad0_rxctrlack1_i,
        QUAD0_RXCTRLACK2_OUT            =>      quad0_rxctrlack2_i,
        QUAD0_RXCTRLACK3_OUT            =>      quad0_rxctrlack3_i,
        QUAD0_RXDATA0_OUT               =>      quad0_rxdata0_i,
        QUAD0_RXDATA1_OUT               =>      quad0_rxdata1_i,
        QUAD0_RXDATA2_OUT               =>      quad0_rxdata2_i,
        QUAD0_RXDATA3_OUT               =>      quad0_rxdata3_i,
        QUAD0_RXN0_IN                   =>      RXN_GTH(0),
        QUAD0_RXN1_IN                   =>      RXN_GTH(1),
        QUAD0_RXN2_IN                   =>      RXN_GTH(2),
        QUAD0_RXN3_IN                   =>      RXN_GTH(3),
        QUAD0_RXP0_IN                   =>      RXP_GTH(0),
        QUAD0_RXP1_IN                   =>      RXP_GTH(1),
        QUAD0_RXP2_IN                   =>      RXP_GTH(2),
        QUAD0_RXP3_IN                   =>      RXP_GTH(3),
        QUAD0_RXSLIP0_IN                =>      quad0_rxslip0_i,
        QUAD0_RXSLIP1_IN                =>      quad0_rxslip1_i,
        QUAD0_RXSLIP2_IN                =>      quad0_rxslip2_i,
        QUAD0_RXSLIP3_IN                =>      quad0_rxslip3_i,
        QUAD0_RXUSERCLKIN0_IN           =>      quad0_rxuserclkin0_i,
        QUAD0_RXUSERCLKIN1_IN           =>      quad0_rxuserclkin0_i,
        QUAD0_RXUSERCLKIN2_IN           =>      quad0_rxuserclkin0_i,
        QUAD0_RXUSERCLKIN3_IN           =>      quad0_rxuserclkin0_i,
        QUAD0_RXUSERCLKOUT0_OUT         =>      quad0_rxuserclkout0_i,
        QUAD0_RXUSERCLKOUT1_OUT         =>      quad0_rxuserclkout1_i,
        QUAD0_RXUSERCLKOUT2_OUT         =>      quad0_rxuserclkout2_i,
        QUAD0_RXUSERCLKOUT3_OUT         =>      quad0_rxuserclkout3_i,
        ------------- Shared Ports - Dynamic Reconfiguration Port (DRP) ------------
        QUAD0_DADDR_IN                  =>      quad0_daddr_i,
        QUAD0_DCLK_IN                   =>      drp_clk_in_i,
        QUAD0_DEN_IN                    =>      quad0_den_i,
        QUAD0_DI_IN                     =>      quad0_di_i,
        QUAD0_DISABLEDRP_IN             =>      quad0_disabledrp_i,
        QUAD0_DRDY_OUT                  =>      quad0_drdy_i,
        QUAD0_DRPDO_OUT                 =>      quad0_drpdo_i,
        QUAD0_DWE_IN                    =>      quad0_dwe_i,
        ---------------------------- Shared Ports - Other --------------------------
        QUAD0_GTHINIT_IN                =>      quad0_gthinit_i,
        QUAD0_GTHRESET_IN               =>      quad0_gthreset_i,
        QUAD0_MGMTPCSLANESEL_IN         =>      quad0_mgmtpcslanesel_i,
        QUAD0_MGMTPCSMMDADDR_IN         =>      quad0_mgmtpcsmmdaddr_i,
        QUAD0_MGMTPCSRDACK_OUT          =>      quad0_mgmtpcsrdack_i,
        QUAD0_MGMTPCSRDDATA_OUT         =>      quad0_mgmtpcsrddata_i,
        QUAD0_MGMTPCSREGADDR_IN         =>      quad0_mgmtpcsregaddr_i,
        QUAD0_MGMTPCSREGRD_IN           =>      quad0_mgmtpcsregrd_i,
        QUAD0_MGMTPCSREGWR_IN           =>      quad0_mgmtpcsregwr_i,
        QUAD0_MGMTPCSWRDATA_IN          =>      quad0_mgmtpcswrdata_i,
        QUAD0_REFCLK_IN                 =>      quad0_refclk_i,
        QUAD0_REFCLK_OUT                =>      quad0_refclk_o,
        ------------------------------- Transmit Ports -----------------------------
        QUAD0_TXBUFRESET0_IN            =>      quad0_txbufreset0_i,
        QUAD0_TXBUFRESET1_IN            =>      quad0_txbufreset1_i,
        QUAD0_TXBUFRESET2_IN            =>      quad0_txbufreset2_i,
        QUAD0_TXBUFRESET3_IN            =>      quad0_txbufreset3_i,
        QUAD0_TXCTRLACK0_OUT            =>      quad0_txctrlack0_i,
        QUAD0_TXCTRLACK1_OUT            =>      quad0_txctrlack1_i,
        QUAD0_TXCTRLACK2_OUT            =>      quad0_txctrlack2_i,
        QUAD0_TXCTRLACK3_OUT            =>      quad0_txctrlack3_i,
        QUAD0_TXDATA0_IN                =>      quad0_txdata0_i,
        QUAD0_TXDATA1_IN                =>      quad0_txdata1_i,
        QUAD0_TXDATA2_IN                =>      quad0_txdata2_i,
        QUAD0_TXDATA3_IN                =>      quad0_txdata3_i,
        QUAD0_TXN0_OUT                  =>      TXN_GTH(0),
        QUAD0_TXN1_OUT                  =>      TXN_GTH(1),
        QUAD0_TXN2_OUT                  =>      TXN_GTH(2),
        QUAD0_TXN3_OUT                  =>      TXN_GTH(3),
        QUAD0_TXP0_OUT                  =>      TXP_GTH(0),
        QUAD0_TXP1_OUT                  =>      TXP_GTH(1),
        QUAD0_TXP2_OUT                  =>      TXP_GTH(2),
        QUAD0_TXP3_OUT                  =>      TXP_GTH(3),
        QUAD0_TXUSERCLKIN0_IN           =>      quad0_txuserclkin0_i,
        QUAD0_TXUSERCLKIN1_IN           =>      quad0_txuserclkin0_i,
        QUAD0_TXUSERCLKIN2_IN           =>      quad0_txuserclkin0_i,
        QUAD0_TXUSERCLKIN3_IN           =>      quad0_txuserclkin0_i,
        QUAD0_TXUSERCLKOUT0_OUT         =>      quad0_txuserclkout0_i,
        QUAD0_TXUSERCLKOUT1_OUT         =>      quad0_txuserclkout1_i,
        QUAD0_TXUSERCLKOUT2_OUT         =>      quad0_txuserclkout2_i,
        QUAD0_TXUSERCLKOUT3_OUT         =>      quad0_txuserclkout3_i,


        --_____________________________________________________________________
        --_____________________________________________________________________
        --QUAD1  (X1Y1)
        QUAD1_TX_PCS_RESETDONE0_OUT     =>      quad1_tx_pcs_resetdone0_i,
        QUAD1_RX_PCS_CDR_RESETDONE0_OUT =>      quad1_rx_pcs_cdr_resetdone0_i,
        QUAD1_TX_PCS_RESET0_IN          =>      quad1_tx_pcs_reset0_i,
        QUAD1_RX_PCS_CDR_RESET0_IN      =>      quad1_rx_pcs_cdr_reset0_i,
        QUAD1_TX_PCS_RESETDONE1_OUT     =>      quad1_tx_pcs_resetdone1_i,
        QUAD1_RX_PCS_CDR_RESETDONE1_OUT =>      quad1_rx_pcs_cdr_resetdone1_i,
        QUAD1_TX_PCS_RESET1_IN          =>      quad1_tx_pcs_reset1_i,
        QUAD1_RX_PCS_CDR_RESET1_IN      =>      quad1_rx_pcs_cdr_reset1_i,
        QUAD1_TX_PCS_RESETDONE2_OUT     =>      quad1_tx_pcs_resetdone2_i,
        QUAD1_RX_PCS_CDR_RESETDONE2_OUT =>      quad1_rx_pcs_cdr_resetdone2_i,
        QUAD1_TX_PCS_RESET2_IN          =>      quad1_tx_pcs_reset2_i,
        QUAD1_RX_PCS_CDR_RESET2_IN      =>      quad1_rx_pcs_cdr_reset2_i,
        QUAD1_TX_PCS_RESETDONE3_OUT     =>      quad1_tx_pcs_resetdone3_i,
        QUAD1_RX_PCS_CDR_RESETDONE3_OUT =>      quad1_rx_pcs_cdr_resetdone3_i,
        QUAD1_TX_PCS_RESET3_IN          =>      quad1_tx_pcs_reset3_i,
        QUAD1_RX_PCS_CDR_RESET3_IN      =>      quad1_rx_pcs_cdr_reset3_i,
        QUAD1_GTHINITDONE_OUT           =>      quad1_gthinitdone_i, --open,
        --------------------------------- Powerdown --------------------------------
        QUAD1_POWERDOWN0_IN             =>      quad1_powerdown0_i,
        QUAD1_POWERDOWN1_IN             =>      quad1_powerdown1_i,
        QUAD1_POWERDOWN2_IN             =>      quad1_powerdown2_i,
        QUAD1_POWERDOWN3_IN             =>      quad1_powerdown3_i,
        QUAD1_RXPOWERDOWN0_IN           =>      quad1_rxpowerdown0_i,
        QUAD1_RXPOWERDOWN1_IN           =>      quad1_rxpowerdown1_i,
        QUAD1_RXPOWERDOWN2_IN           =>      quad1_rxpowerdown2_i,
        QUAD1_RXPOWERDOWN3_IN           =>      quad1_rxpowerdown3_i,
        QUAD1_TXPOWERDOWN0_IN           =>      quad1_txpowerdown0_i,
        QUAD1_TXPOWERDOWN1_IN           =>      quad1_txpowerdown1_i,
        QUAD1_TXPOWERDOWN2_IN           =>      quad1_txpowerdown2_i,
        QUAD1_TXPOWERDOWN3_IN           =>      quad1_txpowerdown3_i,
        ------------------------------- Receive Ports ------------------------------
        QUAD1_RXBUFRESET0_IN            =>      quad1_rxbufreset0_i,
        QUAD1_RXBUFRESET1_IN            =>      quad1_rxbufreset1_i,
        QUAD1_RXBUFRESET2_IN            =>      quad1_rxbufreset2_i,
        QUAD1_RXBUFRESET3_IN            =>      quad1_rxbufreset3_i,
        QUAD1_RXCTRLACK0_OUT            =>      quad1_rxctrlack0_i,
        QUAD1_RXCTRLACK1_OUT            =>      quad1_rxctrlack1_i,
        QUAD1_RXCTRLACK2_OUT            =>      quad1_rxctrlack2_i,
        QUAD1_RXCTRLACK3_OUT            =>      quad1_rxctrlack3_i,
        QUAD1_RXDATA0_OUT               =>      quad1_rxdata0_i,
        QUAD1_RXDATA1_OUT               =>      quad1_rxdata1_i,
        QUAD1_RXDATA2_OUT               =>      quad1_rxdata2_i,
        QUAD1_RXDATA3_OUT               =>      quad1_rxdata3_i,
        QUAD1_RXN0_IN                   =>      RXN_GTH(4),
        QUAD1_RXN1_IN                   =>      RXN_GTH(5),
        QUAD1_RXN2_IN                   =>      RXN_GTH(6),
        QUAD1_RXN3_IN                   =>      RXN_GTH(7),
        QUAD1_RXP0_IN                   =>      RXP_GTH(4),
        QUAD1_RXP1_IN                   =>      RXP_GTH(5),
        QUAD1_RXP2_IN                   =>      RXP_GTH(6),
        QUAD1_RXP3_IN                   =>      RXP_GTH(7),
        QUAD1_RXSLIP0_IN                =>      quad1_rxslip0_i,
        QUAD1_RXSLIP1_IN                =>      quad1_rxslip1_i,
        QUAD1_RXSLIP2_IN                =>      quad1_rxslip2_i,
        QUAD1_RXSLIP3_IN                =>      quad1_rxslip3_i,
        QUAD1_RXUSERCLKIN0_IN           =>      quad0_rxuserclkin0_i,
        QUAD1_RXUSERCLKIN1_IN           =>      quad0_rxuserclkin0_i,
        QUAD1_RXUSERCLKIN2_IN           =>      quad0_rxuserclkin0_i,
        QUAD1_RXUSERCLKIN3_IN           =>      quad0_rxuserclkin0_i,
        QUAD1_RXUSERCLKOUT0_OUT         =>      quad1_rxuserclkout0_i,
        QUAD1_RXUSERCLKOUT1_OUT         =>      quad1_rxuserclkout1_i,
        QUAD1_RXUSERCLKOUT2_OUT         =>      quad1_rxuserclkout2_i,
        QUAD1_RXUSERCLKOUT3_OUT         =>      quad1_rxuserclkout3_i,
        ------------- Shared Ports - Dynamic Reconfiguration Port (DRP) ------------
        QUAD1_DADDR_IN                  =>      quad1_daddr_i,
        QUAD1_DCLK_IN                   =>      drp_clk_in_i,
        QUAD1_DEN_IN                    =>      quad1_den_i,
        QUAD1_DI_IN                     =>      quad1_di_i,
        QUAD1_DISABLEDRP_IN             =>      quad1_disabledrp_i,
        QUAD1_DRDY_OUT                  =>      quad1_drdy_i,
        QUAD1_DRPDO_OUT                 =>      quad1_drpdo_i,
        QUAD1_DWE_IN                    =>      quad1_dwe_i,
        ---------------------------- Shared Ports - Other --------------------------
        QUAD1_GTHINIT_IN                =>      quad1_gthinit_i,
        QUAD1_GTHRESET_IN               =>      quad1_gthreset_i,
        QUAD1_MGMTPCSLANESEL_IN         =>      quad1_mgmtpcslanesel_i,
        QUAD1_MGMTPCSMMDADDR_IN         =>      quad1_mgmtpcsmmdaddr_i,
        QUAD1_MGMTPCSRDACK_OUT          =>      quad1_mgmtpcsrdack_i,
        QUAD1_MGMTPCSRDDATA_OUT         =>      quad1_mgmtpcsrddata_i,
        QUAD1_MGMTPCSREGADDR_IN         =>      quad1_mgmtpcsregaddr_i,
        QUAD1_MGMTPCSREGRD_IN           =>      quad1_mgmtpcsregrd_i,
        QUAD1_MGMTPCSREGWR_IN           =>      quad1_mgmtpcsregwr_i,
        QUAD1_MGMTPCSWRDATA_IN          =>      quad1_mgmtpcswrdata_i,
        QUAD1_REFCLK_IN                 =>      quad1_refclk_i,
        ------------------------------- Transmit Ports -----------------------------
        QUAD1_TXBUFRESET0_IN            =>      quad1_txbufreset0_i,
        QUAD1_TXBUFRESET1_IN            =>      quad1_txbufreset1_i,
        QUAD1_TXBUFRESET2_IN            =>      quad1_txbufreset2_i,
        QUAD1_TXBUFRESET3_IN            =>      quad1_txbufreset3_i,
        QUAD1_TXCTRLACK0_OUT            =>      quad1_txctrlack0_i,
        QUAD1_TXCTRLACK1_OUT            =>      quad1_txctrlack1_i,
        QUAD1_TXCTRLACK2_OUT            =>      quad1_txctrlack2_i,
        QUAD1_TXCTRLACK3_OUT            =>      quad1_txctrlack3_i,
        QUAD1_TXDATA0_IN                =>      quad1_txdata0_i,
        QUAD1_TXDATA1_IN                =>      quad1_txdata1_i,
        QUAD1_TXDATA2_IN                =>      quad1_txdata2_i,
        QUAD1_TXDATA3_IN                =>      quad1_txdata3_i,
        QUAD1_TXN0_OUT                  =>      TXN_GTH(4),
        QUAD1_TXN1_OUT                  =>      TXN_GTH(5),
        QUAD1_TXN2_OUT                  =>      TXN_GTH(6),
        QUAD1_TXN3_OUT                  =>      TXN_GTH(7),
        QUAD1_TXP0_OUT                  =>      TXP_GTH(4),
        QUAD1_TXP1_OUT                  =>      TXP_GTH(5),
        QUAD1_TXP2_OUT                  =>      TXP_GTH(6),
        QUAD1_TXP3_OUT                  =>      TXP_GTH(7),
        QUAD1_TXUSERCLKIN0_IN           =>      quad0_txuserclkin0_i,
        QUAD1_TXUSERCLKIN1_IN           =>      quad0_txuserclkin0_i,
        QUAD1_TXUSERCLKIN2_IN           =>      quad0_txuserclkin0_i,
        QUAD1_TXUSERCLKIN3_IN           =>      quad0_txuserclkin0_i,
        QUAD1_TXUSERCLKOUT0_OUT         =>      quad1_txuserclkout0_i,
        QUAD1_TXUSERCLKOUT1_OUT         =>      quad1_txuserclkout1_i,
        QUAD1_TXUSERCLKOUT2_OUT         =>      quad1_txuserclkout2_i,
        QUAD1_TXUSERCLKOUT3_OUT         =>      quad1_txuserclkout3_i,

        --_____________________________________________________________________
        --_____________________________________________________________________
        --QUAD2  (X1Y2)
        QUAD2_TX_PCS_RESETDONE0_OUT     =>      quad2_tx_pcs_resetdone0_i,
        QUAD2_RX_PCS_CDR_RESETDONE0_OUT =>      quad2_rx_pcs_cdr_resetdone0_i,
        QUAD2_TX_PCS_RESET0_IN          =>      quad2_tx_pcs_reset0_i,
        QUAD2_RX_PCS_CDR_RESET0_IN      =>      quad2_rx_pcs_cdr_reset0_i,
        QUAD2_TX_PCS_RESETDONE1_OUT     =>      quad2_tx_pcs_resetdone1_i,
        QUAD2_RX_PCS_CDR_RESETDONE1_OUT =>      quad2_rx_pcs_cdr_resetdone1_i,
        QUAD2_TX_PCS_RESET1_IN          =>      quad2_tx_pcs_reset1_i,
        QUAD2_RX_PCS_CDR_RESET1_IN      =>      quad2_rx_pcs_cdr_reset1_i,
        QUAD2_TX_PCS_RESETDONE2_OUT     =>      quad2_tx_pcs_resetdone2_i,
        QUAD2_RX_PCS_CDR_RESETDONE2_OUT =>      quad2_rx_pcs_cdr_resetdone2_i,
        QUAD2_TX_PCS_RESET2_IN          =>      quad2_tx_pcs_reset2_i,
        QUAD2_RX_PCS_CDR_RESET2_IN      =>      quad2_rx_pcs_cdr_reset2_i,
        QUAD2_TX_PCS_RESETDONE3_OUT     =>      quad2_tx_pcs_resetdone3_i,
        QUAD2_RX_PCS_CDR_RESETDONE3_OUT =>      quad2_rx_pcs_cdr_resetdone3_i,
        QUAD2_TX_PCS_RESET3_IN          =>      quad2_tx_pcs_reset3_i,
        QUAD2_RX_PCS_CDR_RESET3_IN      =>      quad2_rx_pcs_cdr_reset3_i,
        QUAD2_GTHINITDONE_OUT           =>      quad2_gthinitdone_i, --open,
        --------------------------------- Powerdown --------------------------------
        QUAD2_POWERDOWN0_IN             =>      quad2_powerdown0_i,
        QUAD2_POWERDOWN1_IN             =>      quad2_powerdown1_i,
        QUAD2_POWERDOWN2_IN             =>      quad2_powerdown2_i,
        QUAD2_POWERDOWN3_IN             =>      quad2_powerdown3_i,
        QUAD2_RXPOWERDOWN0_IN           =>      quad2_rxpowerdown0_i,
        QUAD2_RXPOWERDOWN1_IN           =>      quad2_rxpowerdown1_i,
        QUAD2_RXPOWERDOWN2_IN           =>      quad2_rxpowerdown2_i,
        QUAD2_RXPOWERDOWN3_IN           =>      quad2_rxpowerdown3_i,
        QUAD2_TXPOWERDOWN0_IN           =>      quad2_txpowerdown0_i,
        QUAD2_TXPOWERDOWN1_IN           =>      quad2_txpowerdown1_i,
        QUAD2_TXPOWERDOWN2_IN           =>      quad2_txpowerdown2_i,
        QUAD2_TXPOWERDOWN3_IN           =>      quad2_txpowerdown3_i,
        ------------------------------- Receive Ports ------------------------------
        QUAD2_RXBUFRESET0_IN            =>      quad2_rxbufreset0_i,
        QUAD2_RXBUFRESET1_IN            =>      quad2_rxbufreset1_i,
        QUAD2_RXBUFRESET2_IN            =>      quad2_rxbufreset2_i,
        QUAD2_RXBUFRESET3_IN            =>      quad2_rxbufreset3_i,
        QUAD2_RXCTRLACK0_OUT            =>      quad2_rxctrlack0_i,
        QUAD2_RXCTRLACK1_OUT            =>      quad2_rxctrlack1_i,
        QUAD2_RXCTRLACK2_OUT            =>      quad2_rxctrlack2_i,
        QUAD2_RXCTRLACK3_OUT            =>      quad2_rxctrlack3_i,
        QUAD2_RXDATA0_OUT               =>      quad2_rxdata0_i,
        QUAD2_RXDATA1_OUT               =>      quad2_rxdata1_i,
        QUAD2_RXDATA2_OUT               =>      quad2_rxdata2_i,
        QUAD2_RXDATA3_OUT               =>      quad2_rxdata3_i,
        QUAD2_RXN0_IN                   =>      RXN_GTH(8),
        QUAD2_RXN1_IN                   =>      RXN_GTH(9),
        QUAD2_RXN2_IN                   =>      RXN_GTH(10),
        QUAD2_RXN3_IN                   =>      RXN_GTH(11),
        QUAD2_RXP0_IN                   =>      RXP_GTH(8),
        QUAD2_RXP1_IN                   =>      RXP_GTH(9),
        QUAD2_RXP2_IN                   =>      RXP_GTH(10),
        QUAD2_RXP3_IN                   =>      RXP_GTH(11),
        QUAD2_RXSLIP0_IN                =>      quad2_rxslip0_i,
        QUAD2_RXSLIP1_IN                =>      quad2_rxslip1_i,
        QUAD2_RXSLIP2_IN                =>      quad2_rxslip2_i,
        QUAD2_RXSLIP3_IN                =>      quad2_rxslip3_i,
        QUAD2_RXUSERCLKIN0_IN           =>      quad0_rxuserclkin0_i,
        QUAD2_RXUSERCLKIN1_IN           =>      quad0_rxuserclkin0_i,
        QUAD2_RXUSERCLKIN2_IN           =>      quad0_rxuserclkin0_i,
        QUAD2_RXUSERCLKIN3_IN           =>      quad0_rxuserclkin0_i,
        QUAD2_RXUSERCLKOUT0_OUT         =>      quad2_rxuserclkout0_i,
        QUAD2_RXUSERCLKOUT1_OUT         =>      quad2_rxuserclkout1_i,
        QUAD2_RXUSERCLKOUT2_OUT         =>      quad2_rxuserclkout2_i,
        QUAD2_RXUSERCLKOUT3_OUT         =>      quad2_rxuserclkout3_i,
        ------------- Shared Ports - Dynamic Reconfiguration Port (DRP) ------------
        QUAD2_DADDR_IN                  =>      quad2_daddr_i,
        QUAD2_DCLK_IN                   =>      drp_clk_in_i,
        QUAD2_DEN_IN                    =>      quad2_den_i,
        QUAD2_DI_IN                     =>      quad2_di_i,
        QUAD2_DISABLEDRP_IN             =>      quad2_disabledrp_i,
        QUAD2_DRDY_OUT                  =>      quad2_drdy_i,
        QUAD2_DRPDO_OUT                 =>      quad2_drpdo_i,
        QUAD2_DWE_IN                    =>      quad2_dwe_i,
        ---------------------------- Shared Ports - Other --------------------------
        QUAD2_GTHINIT_IN                =>      quad2_gthinit_i,
        QUAD2_GTHRESET_IN               =>      quad2_gthreset_i,
        QUAD2_MGMTPCSLANESEL_IN         =>      quad2_mgmtpcslanesel_i,
        QUAD2_MGMTPCSMMDADDR_IN         =>      quad2_mgmtpcsmmdaddr_i,
        QUAD2_MGMTPCSRDACK_OUT          =>      quad2_mgmtpcsrdack_i,
        QUAD2_MGMTPCSRDDATA_OUT         =>      quad2_mgmtpcsrddata_i,
        QUAD2_MGMTPCSREGADDR_IN         =>      quad2_mgmtpcsregaddr_i,
        QUAD2_MGMTPCSREGRD_IN           =>      quad2_mgmtpcsregrd_i,
        QUAD2_MGMTPCSREGWR_IN           =>      quad2_mgmtpcsregwr_i,
        QUAD2_MGMTPCSWRDATA_IN          =>      quad2_mgmtpcswrdata_i,
        QUAD2_REFCLK_IN                 =>      quad2_refclk_i,
        ------------------------------- Transmit Ports -----------------------------
        QUAD2_TXBUFRESET0_IN            =>      quad2_txbufreset0_i,
        QUAD2_TXBUFRESET1_IN            =>      quad2_txbufreset1_i,
        QUAD2_TXBUFRESET2_IN            =>      quad2_txbufreset2_i,
        QUAD2_TXBUFRESET3_IN            =>      quad2_txbufreset3_i,
        QUAD2_TXCTRLACK0_OUT            =>      quad2_txctrlack0_i,
        QUAD2_TXCTRLACK1_OUT            =>      quad2_txctrlack1_i,
        QUAD2_TXCTRLACK2_OUT            =>      quad2_txctrlack2_i,
        QUAD2_TXCTRLACK3_OUT            =>      quad2_txctrlack3_i,
        QUAD2_TXDATA0_IN                =>      quad2_txdata0_i,
        QUAD2_TXDATA1_IN                =>      quad2_txdata1_i,
        QUAD2_TXDATA2_IN                =>      quad2_txdata2_i,
        QUAD2_TXDATA3_IN                =>      quad2_txdata3_i,
        QUAD2_TXN0_OUT                  =>      TXN_GTH(8),
        QUAD2_TXN1_OUT                  =>      TXN_GTH(9),
        QUAD2_TXN2_OUT                  =>      TXN_GTH(10),
        QUAD2_TXN3_OUT                  =>      TXN_GTH(11),
        QUAD2_TXP0_OUT                  =>      TXP_GTH(8),
        QUAD2_TXP1_OUT                  =>      TXP_GTH(9),
        QUAD2_TXP2_OUT                  =>      TXP_GTH(10),
        QUAD2_TXP3_OUT                  =>      TXP_GTH(11),
        QUAD2_TXUSERCLKIN0_IN           =>      quad0_txuserclkin0_i,
        QUAD2_TXUSERCLKIN1_IN           =>      quad0_txuserclkin0_i,
        QUAD2_TXUSERCLKIN2_IN           =>      quad0_txuserclkin0_i,
        QUAD2_TXUSERCLKIN3_IN           =>      quad0_txuserclkin0_i,
        QUAD2_TXUSERCLKOUT0_OUT         =>      quad2_txuserclkout0_i,
        QUAD2_TXUSERCLKOUT1_OUT         =>      quad2_txuserclkout1_i,
        QUAD2_TXUSERCLKOUT2_OUT         =>      quad2_txuserclkout2_i,
        QUAD2_TXUSERCLKOUT3_OUT         =>      quad2_txuserclkout3_i

    );
  
end pcs_phy_100G_arch;

