--
-- Copyright (c) 2015 Aitia International Inc.
--
-- See the file "/doc/license.txt" for information on usage and redistribution
-- of this file, and for a DISCLAIMER OF ALL WARRANTIES.
-- 
--------------------------------------------------------------------------------
-- Company: Aitia
-- Engineer: Laszlo Kovacs
--
-- Create Date:   08:05:16 06/03/2016
-- Design Name:   
-- Module Name:   /mnt/Adat2/MUNKA/SGA_CGEP/sim/sim_100G_core.vhd
-- Project Name:  sim
-- Target Device:  
-- Tool versions:  
-- Description:   
-- 
-- TestBench for 100G Ethernet PCS/PMA+MAC Core module simulation only
-- Contains a simple traffic geneartor
-- Parallel PMA core input/outputs are in loopback!
-- 
-- Dependencies:
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
--
-- Notes: 
-- This testbench has been automatically generated using types std_logic and
-- std_logic_vector for the ports of the unit under test.  Xilinx recommends
-- that these types always be used for the top-level I/O of a design in order
-- to guarantee that the testbench will bind correctly to the post-implementation 
-- simulation model.
--------------------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;
USE ieee.std_logic_unsigned.all;
USE work.tuw.all;
USE work.cg_framegen_pkg.all;

ENTITY sim_100G_core IS
END sim_100G_core;

ARCHITECTURE behavior OF sim_100G_core IS 

--
COMPONENT CORE_100G is
  port (
    Clk_GTH_Rx 		: in STD_LOGIC; 
    Clk_GTH_Tx 		: in STD_LOGIC; 
    Clk_MAC_RX 		: in STD_LOGIC; 
    Clk_MAC_TX 		: in STD_LOGIC; 
    CG_TXDv 			: in STD_LOGIC; 
    CG_TXSof 			: in STD_LOGIC; 
    CG_TXEof 			: in STD_LOGIC; 
    CG_Rst 			: in STD_LOGIC; 
    CG_signal_ok 		: in STD_LOGIC; 
    CG_RXDv 			: out STD_LOGIC; 
    CG_RXSof 			: out STD_LOGIC; 
    CG_RXEof 			: out STD_LOGIC; 
    CG_RXCRCErr 		: out STD_LOGIC; 
    CG_RXErr 			: out STD_LOGIC; 
    CG_TXAv 			: out STD_LOGIC; 
    PCS_RXSof 			: out STD_LOGIC; 
    CG_locked66b 		: out STD_LOGIC; 
    CG_aligned 		: out STD_LOGIC; 
    DOUT_GTH 			: in STD_LOGIC_VECTOR ( 639 downto 0 ); 
    CG_TXData 			: in STD_LOGIC_VECTOR ( 511 downto 0 ); 
    CG_TXMod64 		: in STD_LOGIC_VECTOR ( 5 downto 0 ); 
    DIN_GTH 			: out STD_LOGIC_VECTOR ( 639 downto 0 ); 
    CG_RXData 			: out STD_LOGIC_VECTOR ( 511 downto 0 ); 
    CG_RXMod64 		: out STD_LOGIC_VECTOR ( 5 downto 0 ) 
  );
END COMPONENT;
--

SIGNAL MACTX_Av		: STD_LOGIC;
SIGNAL MACTX_SOF		: STD_LOGIC;
SIGNAL MACTX_EOF		: STD_LOGIC;
SIGNAL MACTX_DAV		: STD_LOGIC;
SIGNAL MACTX_M40		: STD_LOGIC_VECTOR(6-1 downto 0);
SIGNAL MACTX_DATA		: STD_LOGIC_VECTOR(8*64-1 downto 0);

SIGNAL MACRX_Err	: STD_LOGIC;
SIGNAL MACRX_CRCErr	: STD_LOGIC;
SIGNAL MACRX_SOF	: STD_LOGIC;
SIGNAL MACRX_EOF	: STD_LOGIC;
SIGNAL MACRX_DAV	: STD_LOGIC;
SIGNAL MACRX_M64	: STD_LOGIC_VECTOR(6-1 downto 0);
SIGNAL MACRX_DATA	: STD_LOGIC_VECTOR(8*64-1 downto 0);

SIGNAL Dout_GTH	: STD_LOGIC_VECTOR(10*64-1 downto 0);
SIGNAL Din_GTH		: STD_LOGIC_VECTOR(10*64-1 downto 0);

SIGNAL lanes_aligned: STD_LOGIC;
SIGNAL lane_valid	: STD_LOGIC;

SIGNAL RXPck_Cntr 	: STD_LOGIC_VECTOR(4*8-1 downto 0) := (others => '0');
SIGNAL MACRx_Loss 	: STD_LOGIC := '0';

   --Inputs
   signal clk_GTH : std_logic := '0';
   signal clk_MAC_rx : std_logic := '0';
   signal clk_MAC_tx : std_logic := '0';
   signal Rst : std_logic := '0';
   signal dRst : std_logic := '0';
   signal signal_ok : std_logic := '1'; 
   signal Cfg_Wr : STD_LOGIC;

   -- Clock period definitions
   constant clk_GTH_period : time := 6.2 ns;
   constant clk_MAC_rx_period : time := 3.2 ns;
   constant clk_MAC_tx_period : time := 3.2 ns;
 
          

  BEGIN

-- Parallell PMA level loopback!
Din_GTH		<= Dout_GTH;
--

-- DUT: The core itself
UUT: CORE_100G
	PORT MAP(
	-- 2 double rate clocks 4 MAC (312.5MHz)
	Clk_GTH_Rx 				=> clk_GTH, --: IN  STD_LOGIC;
	Clk_GTH_Tx 				=> clk_GTH, --: IN  STD_LOGIC;
	Clk_MAC_RX				=> clk_MAC_Rx, --: IN  STD_LOGIC;
	Clk_MAC_TX				=> clk_MAC_Tx, --: IN  STD_LOGIC;
	DIN_GTH					=> DOUT_GTH, --: OUT STD_LOGIC_VECTOR(10*64-1 downto 0);
	DOUT_GTH					=> DIN_GTH, --: IN  STD_LOGIC_VECTOR(10*64-1 downto 0);
	-- Rx MAC
	CG_RXData					=> MACRX_Data, --: OUT STD_LOGIC_VECTOR(8*64-1 downto 0);
	CG_RXMod64				=> MACRX_M64, --: OUT STD_LOGIC_VECTOR(6-1 downto 0);	
	CG_RXDv					=> MACRX_DAV, --: OUT STD_LOGIC;
	CG_RXSof					=> MACRX_SOF, --: OUT STD_LOGIC;
	CG_RXEof					=> MACRX_EOF, --: OUT STD_LOGIC;
	CG_RXCRCErr				=> MACRX_CRCErr, --: OUT STD_LOGIC;	
	CG_RXErr					=> MACRX_Err, --: OUT STD_LOGIC;	
	-- Tx MAC
	CG_TXAv					=> MACTX_Av, --: OUT STD_LOGIC;
	CG_TXData					=> MACTX_DATA, --: IN  STD_LOGIC_VECTOR(8*64-1 downto 0);
	CG_TXMod64				=> MACTX_M40, --: IN  STD_LOGIC_VECTOR(6-1 downto 0);	
	CG_TXDv					=> MACTX_DAV, --: IN  STD_LOGIC;
	CG_TXSof					=> MACTX_SOF, --: IN  STD_LOGIC;
	CG_TXEof					=> MACTX_EOF, --: IN  STD_LOGIC;
	-- Etc
	PCS_RXSof 				=> open, --: OUT STD_LOGIC;
	CG_Rst					=> Rst, --: IN  STD_LOGIC;
	CG_signal_ok				=> '1', --: IN  STD_LOGIC;
	CG_locked66b				=> lane_valid, --: OUT STD_LOGIC;
	CG_aligned				=> lanes_aligned --: OUT STD_LOGIC
	);

-- Simple Frame Generator
UCTxG: CG_FrameGen 
	PORT MAP(
	clk 						=> clk_MAC_tx,
	Rst 						=> Rst,
	-- Config
	Cfg_Frlen					=> X"0950", --CGCfg_Frlen, 
	Cfg_Ifslen				=> X"0000", --CGCfg_Ifslen, 
	Cfg_Wr					=> Cfg_Wr,
	--
	CG_TxAv					=> MACTX_Av,
	CG_TXData 				=> MACTX_DATA,
	CG_TXDv 					=> MACTX_DAV,
	CG_TXSof					=> MACTX_SOF,
	CG_TXEof					=> MACTX_EOF,
	CG_TXMod64				=> MACTX_M40
	);

-- Detect frame-loss
URXLoss: PROCESS(clk_MAC_rx)
BEGIN
IF (clk_MAC_rx'event AND clk_MAC_rx = '1') THEN
	IF (MACRX_SOF = '1' AND MACRX_DAV = '1') THEN
		IF (RXPck_Cntr /= MACRX_Data(5*64-1 downto 5*64-32)) THEN -- Packet number is in last 4 byte positions
			RXPck_Cntr	<= MACRX_Data(5*64-1 downto 5*64-32) + X"00000001";
			MACRX_Loss 	<= '1';
		ELSE
			INCR(RXPck_Cntr);
			MACRX_Loss 	<= '0';
		END IF;
	ELSE
		MACRX_Loss 	<= '0';
	END IF;
END IF;
END PROCESS;

UTXgen: PROCESS(clk_MAC_tx)
BEGIN
IF (clk_MAC_tx'event AND clk_MAC_tx = '1') THEN
dRst		<= Rst;
	IF (Rst = '0' AND dRst = '1') THEN
		Cfg_Wr		<= '1';
	ELSE
		Cfg_Wr		<= '0';
	END IF;
END IF;
END PROCESS;

  -- Clock process definitions
   clk_GTH_process :process
   begin
		clk_GTH <= '0';
		wait for clk_GTH_period/2;
		clk_GTH <= '1';
		wait for clk_GTH_period/2;
   end process;
 
   clk_MAC_rx_process :process
   begin
		clk_MAC_rx <= '0';
		wait for clk_MAC_rx_period/2;
		clk_MAC_rx <= '1';
		wait for clk_MAC_rx_period/2;
   end process;
 
   clk_MAC_tx_process :process
   begin
		clk_MAC_tx <= '0';
		wait for clk_MAC_tx_period/2;
		clk_MAC_tx <= '1';
		wait for clk_MAC_tx_period/2;
   end process;

   -- Stimulus process
   stim_proc: process
   begin		
      -- hold reset state for 100 ns.
	Rst	<= '1';
      wait for 100 ns;	
	Rst	<= '0';
      wait;
   end process; 

  END;
